# https://pyinstaller.org/en/stable/hooks.html

from _typeshed import StrOrBytesPath, StrPath, SupportsKeysAndGetItem
from collections.abc import Callable, Iterable, Mapping
from typing import Any
from typing_extensions import Literal, TypeAlias

import pkg_resources
from PyInstaller import HOMEPATH as HOMEPATH
from PyInstaller.depend.imphookapi import PostGraphAPI
from PyInstaller.utils.hooks import conda
from PyInstaller.utils.hooks.win32 import get_pywin32_module_file_attribute as get_pywin32_module_file_attribute

conda_support = conda

_Environ: TypeAlias = SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]] | Mapping[str, str]

PY_IGNORE_EXTENSIONS: set[str]
hook_variables: dict[str, str]

def exec_statement(statement: str) -> str | int: ...
def exec_statement_rc(statement: str) -> str | int: ...
def exec_script(script_filename: StrOrBytesPath, *args: str, env: _Environ | None = ...) -> str | int: ...
def exec_script_rc(script_filename: StrOrBytesPath, *args: str, env: _Environ | None = ...) -> str | int: ...
def eval_statement(statement: str) -> Any | Literal[""]: ...
def eval_script(script_filename: StrOrBytesPath, *args: str, env: _Environ | None = ...) -> Any | Literal[""]: ...
def get_pyextension_imports(module_name: str) -> list[str]: ...
def get_homebrew_path(formula: str = ...) -> str | None: ...
def remove_prefix(string: str, prefix: str) -> str: ...
def remove_suffix(string: str, suffix: str) -> str: ...
def remove_file_extension(filename: str) -> str: ...
def can_import_module(module_name: str) -> bool: ...
def get_module_attribute(module_name: str, attr_name: str) -> Any: ...
def get_module_file_attribute(package: str) -> str | None: ...
def is_module_satisfies(
    requirements: Iterable[str] | pkg_resources.Requirement,
    version: str | pkg_resources.Distribution | None = ...,
    version_attr: str = ...,
) -> bool: ...
def is_package(module_name: str) -> bool: ...
def get_all_package_paths(package: str) -> list[str]: ...
def package_base_path(package_path: str, package: str) -> str: ...
def get_package_paths(package: str) -> tuple[str, str]: ...
def collect_submodules(
    package: str, filter: Callable[[str], bool] = ..., on_error: Literal["ignore", "warn once", "warn", "raise"] = ...
) -> list[str]: ...
def is_module_or_submodule(name: str, mod_or_submod: str) -> bool: ...

PY_DYLIB_PATTERNS: list[str]

def collect_dynamic_libs(package: str, destdir: object = ...) -> list[tuple[str, str]]: ...
def collect_data_files(
    package: str,
    include_py_files: bool = ...,
    subdir: StrPath | None = ...,
    excludes: Iterable[str] | None = ...,
    includes: Iterable[str] | None = ...,
) -> list[tuple[str, str]]: ...
def collect_system_data_files(
    path: str, destdir: StrPath | None = ..., include_py_files: bool = ...
) -> list[tuple[str, str]]: ...
def copy_metadata(package_name: str, recursive: bool = ...) -> list[tuple[str, str]]: ...
def get_installer(module: str) -> str | None: ...
def requirements_for_package(package_name: str) -> list[str]: ...
def collect_all(
    package_name: str,
    include_py_files: bool = ...,
    filter_submodules: Callable[[str], bool] | None = ...,
    exclude_datas: Iterable[str] | None = ...,
    include_datas: Iterable[str] | None = ...,
    on_error: Literal["ignore", "warn once", "warn", "raise"] = ...,
) -> tuple[list[tuple[str, str]], list[tuple[str, str]], list[str]]: ...
def collect_entry_point(name: str) -> tuple[tuple[str, str], list[str]]: ...
def get_hook_config(hook_api: PostGraphAPI, module_name: str, key: str) -> None: ...
def include_or_exclude_file(
    filename: StrOrBytesPath,
    include_list: Iterable[StrOrBytesPath] | None = ...,
    exclude_list: Iterable[StrOrBytesPath] | None = ...,
) -> bool: ...
def collect_delvewheel_libs_directory(
    package_name: str,
    libdir_name: StrPath | None = ...,
    datas: list[tuple[str, str]] | None = ...,
    binaries: list[tuple[str, str]] | None = ...,
) -> tuple[list[tuple[str, str]], list[tuple[str, str]]]: ...



from sqlalchemy.connectors.mxodbc import MxODBCConnector
from sqlalchemy.dialects.sybase.base import SybaseDialect, SybaseExecutionContext

class SybaseExecutionContext_mxodbc(SybaseExecutionContext): ...

class SybaseDialect_mxodbc(MxODBCConnector, SybaseDialect):
    supports_statement_cache: bool

dialect = SybaseDialect_mxodbc



ERROR_FIRST: int
NO: int
YES: int
CANT_CREATE_FILE: int
CANT_CREATE_TABLE: int
CANT_CREATE_DB: int
DB_CREATE_EXISTS: int
DB_DROP_EXISTS: int
DB_DROP_RMDIR: int
CANT_FIND_SYSTEM_REC: int
CANT_GET_STAT: int
CANT_LOCK: int
CANT_OPEN_FILE: int
FILE_NOT_FOUND: int
CANT_READ_DIR: int
CHECKREAD: int
DUP_KEY: int
ERROR_ON_READ: int
ERROR_ON_RENAME: int
ERROR_ON_WRITE: int
FILE_USED: int
FILSORT_ABORT: int
GET_ERRNO: int
ILLEGAL_HA: int
KEY_NOT_FOUND: int
NOT_FORM_FILE: int
NOT_KEYFILE: int
OLD_KEYFILE: int
OPEN_AS_READONLY: int
OUTOFMEMORY: int
OUT_OF_SORTMEMORY: int
CON_COUNT_ERROR: int
OUT_OF_RESOURCES: int
BAD_HOST_ERROR: int
HANDSHAKE_ERROR: int
DBACCESS_DENIED_ERROR: int
ACCESS_DENIED_ERROR: int
NO_DB_ERROR: int
UNKNOWN_COM_ERROR: int
BAD_NULL_ERROR: int
BAD_DB_ERROR: int
TABLE_EXISTS_ERROR: int
BAD_TABLE_ERROR: int
NON_UNIQ_ERROR: int
SERVER_SHUTDOWN: int
BAD_FIELD_ERROR: int
WRONG_FIELD_WITH_GROUP: int
WRONG_GROUP_FIELD: int
WRONG_SUM_SELECT: int
WRONG_VALUE_COUNT: int
TOO_LONG_IDENT: int
DUP_FIELDNAME: int
DUP_KEYNAME: int
DUP_ENTRY: int
WRONG_FIELD_SPEC: int
PARSE_ERROR: int
EMPTY_QUERY: int
NONUNIQ_TABLE: int
INVALID_DEFAULT: int
MULTIPLE_PRI_KEY: int
TOO_MANY_KEYS: int
TOO_MANY_KEY_PARTS: int
TOO_LONG_KEY: int
KEY_COLUMN_DOES_NOT_EXITS: int
BLOB_USED_AS_KEY: int
TOO_BIG_FIELDLENGTH: int
WRONG_AUTO_KEY: int
READY: int
SHUTDOWN_COMPLETE: int
FORCING_CLOSE: int
IPSOCK_ERROR: int
NO_SUCH_INDEX: int
WRONG_FIELD_TERMINATORS: int
BLOBS_AND_NO_TERMINATED: int
TEXTFILE_NOT_READABLE: int
FILE_EXISTS_ERROR: int
LOAD_INFO: int
ALTER_INFO: int
WRONG_SUB_KEY: int
CANT_REMOVE_ALL_FIELDS: int
CANT_DROP_FIELD_OR_KEY: int
INSERT_INFO: int
UPDATE_TABLE_USED: int
NO_SUCH_THREAD: int
KILL_DENIED_ERROR: int
NO_TABLES_USED: int
TOO_BIG_SET: int
NO_UNIQUE_LOGFILE: int
TABLE_NOT_LOCKED_FOR_WRITE: int
TABLE_NOT_LOCKED: int
BLOB_CANT_HAVE_DEFAULT: int
WRONG_DB_NAME: int
WRONG_TABLE_NAME: int
TOO_BIG_SELECT: int
UNKNOWN_ERROR: int
UNKNOWN_PROCEDURE: int
WRONG_PARAMCOUNT_TO_PROCEDURE: int
WRONG_PARAMETERS_TO_PROCEDURE: int
UNKNOWN_TABLE: int
FIELD_SPECIFIED_TWICE: int
INVALID_GROUP_FUNC_USE: int
UNSUPPORTED_EXTENSION: int
TABLE_MUST_HAVE_COLUMNS: int
RECORD_FILE_FULL: int
UNKNOWN_CHARACTER_SET: int
TOO_MANY_TABLES: int
TOO_MANY_FIELDS: int
TOO_BIG_ROWSIZE: int
STACK_OVERRUN: int
WRONG_OUTER_JOIN_UNUSED: int
NULL_COLUMN_IN_INDEX: int
CANT_FIND_UDF: int
CANT_INITIALIZE_UDF: int
UDF_NO_PATHS: int
UDF_EXISTS: int
CANT_OPEN_LIBRARY: int
CANT_FIND_DL_ENTRY: int
FUNCTION_NOT_DEFINED: int
HOST_IS_BLOCKED: int
HOST_NOT_PRIVILEGED: int
PASSWORD_ANONYMOUS_USER: int
PASSWORD_NOT_ALLOWED: int
PASSWORD_NO_MATCH: int
UPDATE_INFO: int
CANT_CREATE_THREAD: int
WRONG_VALUE_COUNT_ON_ROW: int
CANT_REOPEN_TABLE: int
INVALID_USE_OF_NULL: int
REGEXP_ERROR: int
MIX_OF_GROUP_FUNC_AND_FIELDS: int
NONEXISTING_GRANT: int
TABLEACCESS_DENIED_ERROR: int
COLUMNACCESS_DENIED_ERROR: int
ILLEGAL_GRANT_FOR_TABLE: int
GRANT_WRONG_HOST_OR_USER: int
NO_SUCH_TABLE: int
NONEXISTING_TABLE_GRANT: int
NOT_ALLOWED_COMMAND: int
SYNTAX_ERROR: int
ABORTING_CONNECTION: int
NET_PACKET_TOO_LARGE: int
NET_READ_ERROR_FROM_PIPE: int
NET_FCNTL_ERROR: int
NET_PACKETS_OUT_OF_ORDER: int
NET_UNCOMPRESS_ERROR: int
NET_READ_ERROR: int
NET_READ_INTERRUPTED: int
NET_ERROR_ON_WRITE: int
NET_WRITE_INTERRUPTED: int
TOO_LONG_STRING: int
TABLE_CANT_HANDLE_BLOB: int
TABLE_CANT_HANDLE_AUTO_INCREMENT: int
WRONG_COLUMN_NAME: int
WRONG_KEY_COLUMN: int
WRONG_MRG_TABLE: int
DUP_UNIQUE: int
BLOB_KEY_WITHOUT_LENGTH: int
PRIMARY_CANT_HAVE_NULL: int
TOO_MANY_ROWS: int
REQUIRES_PRIMARY_KEY: int
UPDATE_WITHOUT_KEY_IN_SAFE_MODE: int
KEY_DOES_NOT_EXITS: int
CHECK_NO_SUCH_TABLE: int
CHECK_NOT_IMPLEMENTED: int
CANT_DO_THIS_DURING_AN_TRANSACTION: int
ERROR_DURING_COMMIT: int
ERROR_DURING_ROLLBACK: int
ERROR_DURING_FLUSH_LOGS: int
NEW_ABORTING_CONNECTION: int
MASTER: int
MASTER_NET_READ: int
MASTER_NET_WRITE: int
FT_MATCHING_KEY_NOT_FOUND: int
LOCK_OR_ACTIVE_TRANSACTION: int
UNKNOWN_SYSTEM_VARIABLE: int
CRASHED_ON_USAGE: int
CRASHED_ON_REPAIR: int
WARNING_NOT_COMPLETE_ROLLBACK: int
TRANS_CACHE_FULL: int
SLAVE_NOT_RUNNING: int
BAD_SLAVE: int
MASTER_INFO: int
SLAVE_THREAD: int
TOO_MANY_USER_CONNECTIONS: int
SET_CONSTANTS_ONLY: int
LOCK_WAIT_TIMEOUT: int
LOCK_TABLE_FULL: int
READ_ONLY_TRANSACTION: int
WRONG_ARGUMENTS: int
NO_PERMISSION_TO_CREATE_USER: int
LOCK_DEADLOCK: int
TABLE_CANT_HANDLE_FT: int
CANNOT_ADD_FOREIGN: int
NO_REFERENCED_ROW: int
ROW_IS_REFERENCED: int
CONNECT_TO_MASTER: int
ERROR_WHEN_EXECUTING_COMMAND: int
WRONG_USAGE: int
WRONG_NUMBER_OF_COLUMNS_IN_SELECT: int
CANT_UPDATE_WITH_READLOCK: int
MIXING_NOT_ALLOWED: int
DUP_ARGUMENT: int
USER_LIMIT_REACHED: int
SPECIFIC_ACCESS_DENIED_ERROR: int
LOCAL_VARIABLE: int
GLOBAL_VARIABLE: int
NO_DEFAULT: int
WRONG_VALUE_FOR_VAR: int
WRONG_TYPE_FOR_VAR: int
VAR_CANT_BE_READ: int
CANT_USE_OPTION_HERE: int
NOT_SUPPORTED_YET: int
MASTER_FATAL_ERROR_READING_BINLOG: int
SLAVE_IGNORED_TABLE: int
INCORRECT_GLOBAL_LOCAL_VAR: int
WRONG_FK_DEF: int
KEY_REF_DO_NOT_MATCH_TABLE_REF: int
OPERAND_COLUMNS: int
SUBQUERY_NO_1_ROW: int
UNKNOWN_STMT_HANDLER: int
CORRUPT_HELP_DB: int
AUTO_CONVERT: int
ILLEGAL_REFERENCE: int
DERIVED_MUST_HAVE_ALIAS: int
SELECT_REDUCED: int
TABLENAME_NOT_ALLOWED_HERE: int
NOT_SUPPORTED_AUTH_MODE: int
SPATIAL_CANT_HAVE_NULL: int
COLLATION_CHARSET_MISMATCH: int
TOO_BIG_FOR_UNCOMPRESS: int
ZLIB_Z_MEM_ERROR: int
ZLIB_Z_BUF_ERROR: int
ZLIB_Z_DATA_ERROR: int
CUT_VALUE_GROUP_CONCAT: int
WARN_TOO_FEW_RECORDS: int
WARN_TOO_MANY_RECORDS: int
WARN_NULL_TO_NOTNULL: int
WARN_DATA_OUT_OF_RANGE: int
WARN_DATA_TRUNCATED: int
WARN_USING_OTHER_HANDLER: int
CANT_AGGREGATE_2COLLATIONS: int
REVOKE_GRANTS: int
CANT_AGGREGATE_3COLLATIONS: int
CANT_AGGREGATE_NCOLLATIONS: int
VARIABLE_IS_NOT_STRUCT: int
UNKNOWN_COLLATION: int
SLAVE_IGNORED_SSL_PARAMS: int
SERVER_IS_IN_SECURE_AUTH_MODE: int
WARN_FIELD_RESOLVED: int
BAD_SLAVE_UNTIL_COND: int
MISSING_SKIP_SLAVE: int
UNTIL_COND_IGNORED: int
WRONG_NAME_FOR_INDEX: int
WRONG_NAME_FOR_CATALOG: int
BAD_FT_COLUMN: int
UNKNOWN_KEY_CACHE: int
WARN_HOSTNAME_WONT_WORK: int
UNKNOWN_STORAGE_ENGINE: int
WARN_DEPRECATED_SYNTAX: int
NON_UPDATABLE_TABLE: int
FEATURE_DISABLED: int
OPTION_PREVENTS_STATEMENT: int
DUPLICATED_VALUE_IN_TYPE: int
TRUNCATED_WRONG_VALUE: int
INVALID_ON_UPDATE: int
UNSUPPORTED_PS: int
GET_ERRMSG: int
GET_TEMPORARY_ERRMSG: int
UNKNOWN_TIME_ZONE: int
WARN_INVALID_TIMESTAMP: int
INVALID_CHARACTER_STRING: int
WARN_ALLOWED_PACKET_OVERFLOWED: int
CONFLICTING_DECLARATIONS: int
SP_NO_RECURSIVE_CREATE: int
SP_ALREADY_EXISTS: int
SP_DOES_NOT_EXIST: int
SP_DROP_FAILED: int
SP_STORE_FAILED: int
SP_LILABEL_MISMATCH: int
SP_LABEL_REDEFINE: int
SP_LABEL_MISMATCH: int
SP_UNINIT_VAR: int
SP_BADSELECT: int
SP_BADRETURN: int
SP_BADSTATEMENT: int
UPDATE_LOG_DEPRECATED_IGNORED: int
UPDATE_LOG_DEPRECATED_TRANSLATED: int
QUERY_INTERRUPTED: int
SP_WRONG_NO_OF_ARGS: int
SP_COND_MISMATCH: int
SP_NORETURN: int
SP_NORETURNEND: int
SP_BAD_CURSOR_QUERY: int
SP_BAD_CURSOR_SELECT: int
SP_CURSOR_MISMATCH: int
SP_CURSOR_ALREADY_OPEN: int
SP_CURSOR_NOT_OPEN: int
SP_UNDECLARED_VAR: int
SP_WRONG_NO_OF_FETCH_ARGS: int
SP_FETCH_NO_DATA: int
SP_DUP_PARAM: int
SP_DUP_VAR: int
SP_DUP_COND: int
SP_DUP_CURS: int
SP_CANT_ALTER: int
SP_SUBSELECT_NYI: int
STMT_NOT_ALLOWED_IN_SF_OR_TRG: int
SP_VARCOND_AFTER_CURSHNDLR: int
SP_CURSOR_AFTER_HANDLER: int
SP_CASE_NOT_FOUND: int
FPARSER_TOO_BIG_FILE: int
FPARSER_BAD_HEADER: int
FPARSER_EOF_IN_COMMENT: int
FPARSER_ERROR_IN_PARAMETER: int
FPARSER_EOF_IN_UNKNOWN_PARAMETER: int
VIEW_NO_EXPLAIN: int
WRONG_OBJECT: int
NONUPDATEABLE_COLUMN: int
VIEW_SELECT_CLAUSE: int
VIEW_SELECT_VARIABLE: int
VIEW_SELECT_TMPTABLE: int
VIEW_WRONG_LIST: int
WARN_VIEW_MERGE: int
WARN_VIEW_WITHOUT_KEY: int
VIEW_INVALID: int
SP_NO_DROP_SP: int
TRG_ALREADY_EXISTS: int
TRG_DOES_NOT_EXIST: int
TRG_ON_VIEW_OR_TEMP_TABLE: int
TRG_CANT_CHANGE_ROW: int
TRG_NO_SUCH_ROW_IN_TRG: int
NO_DEFAULT_FOR_FIELD: int
DIVISION_BY_ZERO: int
TRUNCATED_WRONG_VALUE_FOR_FIELD: int
ILLEGAL_VALUE_FOR_TYPE: int
VIEW_NONUPD_CHECK: int
VIEW_CHECK_FAILED: int
PROCACCESS_DENIED_ERROR: int
RELAY_LOG_FAIL: int
UNKNOWN_TARGET_BINLOG: int
IO_ERR_LOG_INDEX_READ: int
BINLOG_PURGE_PROHIBITED: int
FSEEK_FAIL: int
BINLOG_PURGE_FATAL_ERR: int
LOG_IN_USE: int
LOG_PURGE_UNKNOWN_ERR: int
RELAY_LOG_INIT: int
NO_BINARY_LOGGING: int
RESERVED_SYNTAX: int
PS_MANY_PARAM: int
KEY_PART_0: int
VIEW_CHECKSUM: int
VIEW_MULTIUPDATE: int
VIEW_NO_INSERT_FIELD_LIST: int
VIEW_DELETE_MERGE_VIEW: int
CANNOT_USER: int
XAER_NOTA: int
XAER_INVAL: int
XAER_RMFAIL: int
XAER_OUTSIDE: int
XAER_RMERR: int
XA_RBROLLBACK: int
NONEXISTING_PROC_GRANT: int
PROC_AUTO_GRANT_FAIL: int
PROC_AUTO_REVOKE_FAIL: int
DATA_TOO_LONG: int
SP_BAD_SQLSTATE: int
STARTUP: int
LOAD_FROM_FIXED_SIZE_ROWS_TO_VAR: int
CANT_CREATE_USER_WITH_GRANT: int
WRONG_VALUE_FOR_TYPE: int
TABLE_DEF_CHANGED: int
SP_DUP_HANDLER: int
SP_NOT_VAR_ARG: int
SP_NO_RETSET: int
CANT_CREATE_GEOMETRY_OBJECT: int
BINLOG_UNSAFE_ROUTINE: int
BINLOG_CREATE_ROUTINE_NEED_SUPER: int
STMT_HAS_NO_OPEN_CURSOR: int
COMMIT_NOT_ALLOWED_IN_SF_OR_TRG: int
NO_DEFAULT_FOR_VIEW_FIELD: int
SP_NO_RECURSION: int
TOO_BIG_SCALE: int
TOO_BIG_PRECISION: int
M_BIGGER_THAN_D: int
WRONG_LOCK_OF_SYSTEM_TABLE: int
CONNECT_TO_FOREIGN_DATA_SOURCE: int
QUERY_ON_FOREIGN_DATA_SOURCE: int
FOREIGN_DATA_SOURCE_DOESNT_EXIST: int
FOREIGN_DATA_STRING_INVALID_CANT_CREATE: int
FOREIGN_DATA_STRING_INVALID: int
TRG_IN_WRONG_SCHEMA: int
STACK_OVERRUN_NEED_MORE: int
TOO_LONG_BODY: int
WARN_CANT_DROP_DEFAULT_KEYCACHE: int
TOO_BIG_DISPLAYWIDTH: int
XAER_DUPID: int
DATETIME_FUNCTION_OVERFLOW: int
CANT_UPDATE_USED_TABLE_IN_SF_OR_TRG: int
VIEW_PREVENT_UPDATE: int
PS_NO_RECURSION: int
SP_CANT_SET_AUTOCOMMIT: int
VIEW_FRM_NO_USER: int
VIEW_OTHER_USER: int
NO_SUCH_USER: int
FORBID_SCHEMA_CHANGE: int
ROW_IS_REFERENCED_2: int
NO_REFERENCED_ROW_2: int
SP_BAD_VAR_SHADOW: int
TRG_NO_DEFINER: int
OLD_FILE_FORMAT: int
SP_RECURSION_LIMIT: int
SP_WRONG_NAME: int
TABLE_NEEDS_UPGRADE: int
SP_NO_AGGREGATE: int
MAX_PREPARED_STMT_COUNT_REACHED: int
VIEW_RECURSIVE: int
NON_GROUPING_FIELD_USED: int
TABLE_CANT_HANDLE_SPKEYS: int
NO_TRIGGERS_ON_SYSTEM_SCHEMA: int
REMOVED_SPACES: int
AUTOINC_READ_FAILED: int
USERNAME: int
HOSTNAME: int
WRONG_STRING_LENGTH: int
NON_INSERTABLE_TABLE: int
ADMIN_WRONG_MRG_TABLE: int
TOO_HIGH_LEVEL_OF_NESTING_FOR_SELECT: int
NAME_BECOMES_EMPTY: int
AMBIGUOUS_FIELD_TERM: int
FOREIGN_SERVER_EXISTS: int
FOREIGN_SERVER_DOESNT_EXIST: int
ILLEGAL_HA_CREATE_OPTION: int
PARTITION_REQUIRES_VALUES_ERROR: int
PARTITION_WRONG_VALUES_ERROR: int
PARTITION_MAXVALUE_ERROR: int
PARTITION_WRONG_NO_PART_ERROR: int
PARTITION_WRONG_NO_SUBPART_ERROR: int
WRONG_EXPR_IN_PARTITION_FUNC_ERROR: int
FIELD_NOT_FOUND_PART_ERROR: int
INCONSISTENT_PARTITION_INFO_ERROR: int
PARTITION_FUNC_NOT_ALLOWED_ERROR: int
PARTITIONS_MUST_BE_DEFINED_ERROR: int
RANGE_NOT_INCREASING_ERROR: int
INCONSISTENT_TYPE_OF_FUNCTIONS_ERROR: int
MULTIPLE_DEF_CONST_IN_LIST_PART_ERROR: int
PARTITION_ENTRY_ERROR: int
MIX_HANDLER_ERROR: int
PARTITION_NOT_DEFINED_ERROR: int
TOO_MANY_PARTITIONS_ERROR: int
SUBPARTITION_ERROR: int
CANT_CREATE_HANDLER_FILE: int
BLOB_FIELD_IN_PART_FUNC_ERROR: int
UNIQUE_KEY_NEED_ALL_FIELDS_IN_PF: int
NO_PARTS_ERROR: int
PARTITION_MGMT_ON_NONPARTITIONED: int
FOREIGN_KEY_ON_PARTITIONED: int
DROP_PARTITION_NON_EXISTENT: int
DROP_LAST_PARTITION: int
COALESCE_ONLY_ON_HASH_PARTITION: int
REORG_HASH_ONLY_ON_SAME_NO: int
REORG_NO_PARAM_ERROR: int
ONLY_ON_RANGE_LIST_PARTITION: int
ADD_PARTITION_SUBPART_ERROR: int
ADD_PARTITION_NO_NEW_PARTITION: int
COALESCE_PARTITION_NO_PARTITION: int
REORG_PARTITION_NOT_EXIST: int
SAME_NAME_PARTITION: int
NO_BINLOG_ERROR: int
CONSECUTIVE_REORG_PARTITIONS: int
REORG_OUTSIDE_RANGE: int
PARTITION_FUNCTION_FAILURE: int
LIMITED_PART_RANGE: int
PLUGIN_IS_NOT_LOADED: int
WRONG_VALUE: int
NO_PARTITION_FOR_GIVEN_VALUE: int
FILEGROUP_OPTION_ONLY_ONCE: int
CREATE_FILEGROUP_FAILED: int
DROP_FILEGROUP_FAILED: int
TABLESPACE_AUTO_EXTEND_ERROR: int
WRONG_SIZE_NUMBER: int
SIZE_OVERFLOW_ERROR: int
ALTER_FILEGROUP_FAILED: int
BINLOG_ROW_LOGGING_FAILED: int
EVENT_ALREADY_EXISTS: int
EVENT_DOES_NOT_EXIST: int
EVENT_INTERVAL_NOT_POSITIVE_OR_TOO_BIG: int
EVENT_ENDS_BEFORE_STARTS: int
EVENT_EXEC_TIME_IN_THE_PAST: int
EVENT_SAME_NAME: int
DROP_INDEX_FK: int
WARN_DEPRECATED_SYNTAX_WITH_VER: int
CANT_LOCK_LOG_TABLE: int
FOREIGN_DUPLICATE_KEY_OLD_UNUSED: int
COL_COUNT_DOESNT_MATCH_PLEASE_UPDATE: int
TEMP_TABLE_PREVENTS_SWITCH_OUT_OF_RBR: int
STORED_FUNCTION_PREVENTS_SWITCH_BINLOG_FORMAT: int
PARTITION_NO_TEMPORARY: int
PARTITION_CONST_DOMAIN_ERROR: int
PARTITION_FUNCTION_IS_NOT_ALLOWED: int
NULL_IN_VALUES_LESS_THAN: int
WRONG_PARTITION_NAME: int
CANT_CHANGE_TX_CHARACTERISTICS: int
DUP_ENTRY_AUTOINCREMENT_CASE: int
EVENT_SET_VAR_ERROR: int
PARTITION_MERGE_ERROR: int
BASE64_DECODE_ERROR: int
EVENT_RECURSION_FORBIDDEN: int
ONLY_INTEGERS_ALLOWED: int
UNSUPORTED_LOG_ENGINE: int
BAD_LOG_STATEMENT: int
CANT_RENAME_LOG_TABLE: int
WRONG_PARAMCOUNT_TO_NATIVE_FCT: int
WRONG_PARAMETERS_TO_NATIVE_FCT: int
WRONG_PARAMETERS_TO_STORED_FCT: int
NATIVE_FCT_NAME_COLLISION: int
DUP_ENTRY_WITH_KEY_NAME: int
BINLOG_PURGE_EMFILE: int
EVENT_CANNOT_CREATE_IN_THE_PAST: int
EVENT_CANNOT_ALTER_IN_THE_PAST: int
NO_PARTITION_FOR_GIVEN_VALUE_SILENT: int
BINLOG_UNSAFE_STATEMENT: int
BINLOG_FATAL_ERROR: int
BINLOG_LOGGING_IMPOSSIBLE: int
VIEW_NO_CREATION_CTX: int
VIEW_INVALID_CREATION_CTX: int
TRG_CORRUPTED_FILE: int
TRG_NO_CREATION_CTX: int
TRG_INVALID_CREATION_CTX: int
EVENT_INVALID_CREATION_CTX: int
TRG_CANT_OPEN_TABLE: int
NO_FORMAT_DESCRIPTION_EVENT_BEFORE_BINLOG_STATEMENT: int
SLAVE_CORRUPT_EVENT: int
LOG_PURGE_NO_FILE: int
XA_RBTIMEOUT: int
XA_RBDEADLOCK: int
NEED_REPREPARE: int
WARN_NO_MASTER_INFO: int
WARN_OPTION_IGNORED: int
PLUGIN_DELETE_BUILTIN: int
WARN_PLUGIN_BUSY: int
VARIABLE_IS_READONLY: int
WARN_ENGINE_TRANSACTION_ROLLBACK: int
SLAVE_HEARTBEAT_VALUE_OUT_OF_RANGE: int
NDB_REPLICATION_SCHEMA_ERROR: int
CONFLICT_FN_PARSE_ERROR: int
EXCEPTIONS_WRITE_ERROR: int
TOO_LONG_TABLE_COMMENT: int
TOO_LONG_FIELD_COMMENT: int
FUNC_INEXISTENT_NAME_COLLISION: int
DATABASE_NAME: int
TABLE_NAME: int
PARTITION_NAME: int
SUBPARTITION_NAME: int
TEMPORARY_NAME: int
RENAMED_NAME: int
TOO_MANY_CONCURRENT_TRXS: int
WARN_NON_ASCII_SEPARATOR_NOT_IMPLEMENTED: int
DEBUG_SYNC_TIMEOUT: int
DEBUG_SYNC_HIT_LIMIT: int
DUP_SIGNAL_SET: int
SIGNAL_WARN: int
SIGNAL_NOT_FOUND: int
SIGNAL_EXCEPTION: int
RESIGNAL_WITHOUT_ACTIVE_HANDLER: int
SIGNAL_BAD_CONDITION_TYPE: int
WARN_COND_ITEM_TRUNCATED: int
COND_ITEM_TOO_LONG: int
UNKNOWN_LOCALE: int
SLAVE_IGNORE_SERVER_IDS: int
SAME_NAME_PARTITION_FIELD: int
PARTITION_COLUMN_LIST_ERROR: int
WRONG_TYPE_COLUMN_VALUE_ERROR: int
TOO_MANY_PARTITION_FUNC_FIELDS_ERROR: int
MAXVALUE_IN_VALUES_IN: int
TOO_MANY_VALUES_ERROR: int
ROW_SINGLE_PARTITION_FIELD_ERROR: int
FIELD_TYPE_NOT_ALLOWED_AS_PARTITION_FIELD: int
PARTITION_FIELDS_TOO_LONG: int
BINLOG_ROW_ENGINE_AND_STMT_ENGINE: int
BINLOG_ROW_MODE_AND_STMT_ENGINE: int
BINLOG_UNSAFE_AND_STMT_ENGINE: int
BINLOG_ROW_INJECTION_AND_STMT_ENGINE: int
BINLOG_STMT_MODE_AND_ROW_ENGINE: int
BINLOG_ROW_INJECTION_AND_STMT_MODE: int
BINLOG_MULTIPLE_ENGINES_AND_SELF_LOGGING_ENGINE: int
BINLOG_UNSAFE_LIMIT: int
BINLOG_UNSAFE_SYSTEM_TABLE: int
BINLOG_UNSAFE_AUTOINC_COLUMNS: int
BINLOG_UNSAFE_UDF: int
BINLOG_UNSAFE_SYSTEM_VARIABLE: int
BINLOG_UNSAFE_SYSTEM_FUNCTION: int
BINLOG_UNSAFE_NONTRANS_AFTER_TRANS: int
MESSAGE_AND_STATEMENT: int
SLAVE_CANT_CREATE_CONVERSION: int
INSIDE_TRANSACTION_PREVENTS_SWITCH_BINLOG_FORMAT: int
PATH_LENGTH: int
WARN_DEPRECATED_SYNTAX_NO_REPLACEMENT: int
WRONG_NATIVE_TABLE_STRUCTURE: int
WRONG_PERFSCHEMA_USAGE: int
WARN_I_S_SKIPPED_TABLE: int
INSIDE_TRANSACTION_PREVENTS_SWITCH_BINLOG_DIRECT: int
STORED_FUNCTION_PREVENTS_SWITCH_BINLOG_DIRECT: int
SPATIAL_MUST_HAVE_GEOM_COL: int
TOO_LONG_INDEX_COMMENT: int
LOCK_ABORTED: int
DATA_OUT_OF_RANGE: int
WRONG_SPVAR_TYPE_IN_LIMIT: int
BINLOG_UNSAFE_MULTIPLE_ENGINES_AND_SELF_LOGGING_ENGINE: int
BINLOG_UNSAFE_MIXED_STATEMENT: int
INSIDE_TRANSACTION_PREVENTS_SWITCH_SQL_LOG_BIN: int
STORED_FUNCTION_PREVENTS_SWITCH_SQL_LOG_BIN: int
FAILED_READ_FROM_PAR_FILE: int
VALUES_IS_NOT_INT_TYPE_ERROR: int
ACCESS_DENIED_NO_PASSWORD_ERROR: int
SET_PASSWORD_AUTH_PLUGIN: int
TRUNCATE_ILLEGAL_FK: int
PLUGIN_IS_PERMANENT: int
SLAVE_HEARTBEAT_VALUE_OUT_OF_RANGE_MIN: int
SLAVE_HEARTBEAT_VALUE_OUT_OF_RANGE_MAX: int
STMT_CACHE_FULL: int
MULTI_UPDATE_KEY_CONFLICT: int
TABLE_NEEDS_REBUILD: int
WARN_OPTION_BELOW_LIMIT: int
INDEX_COLUMN_TOO_LONG: int
ERROR_IN_TRIGGER_BODY: int
ERROR_IN_UNKNOWN_TRIGGER_BODY: int
INDEX_CORRUPT: int
UNDO_RECORD_TOO_BIG: int
BINLOG_UNSAFE_INSERT_IGNORE_SELECT: int
BINLOG_UNSAFE_INSERT_SELECT_UPDATE: int
BINLOG_UNSAFE_REPLACE_SELECT: int
BINLOG_UNSAFE_CREATE_IGNORE_SELECT: int
BINLOG_UNSAFE_CREATE_REPLACE_SELECT: int
BINLOG_UNSAFE_UPDATE_IGNORE: int
PLUGIN_NO_UNINSTALL: int
PLUGIN_NO_INSTALL: int
BINLOG_UNSAFE_WRITE_AUTOINC_SELECT: int
BINLOG_UNSAFE_CREATE_SELECT_AUTOINC: int
BINLOG_UNSAFE_INSERT_TWO_KEYS: int
TABLE_IN_FK_CHECK: int
UNSUPPORTED_ENGINE: int
BINLOG_UNSAFE_AUTOINC_NOT_FIRST: int
CANNOT_LOAD_FROM_TABLE_V2: int
MASTER_DELAY_VALUE_OUT_OF_RANGE: int
ONLY_FD_AND_RBR_EVENTS_ALLOWED_IN_BINLOG_STATEMENT: int
PARTITION_EXCHANGE_DIFFERENT_OPTION: int
PARTITION_EXCHANGE_PART_TABLE: int
PARTITION_EXCHANGE_TEMP_TABLE: int
PARTITION_INSTEAD_OF_SUBPARTITION: int
UNKNOWN_PARTITION: int
TABLES_DIFFERENT_METADATA: int
ROW_DOES_NOT_MATCH_PARTITION: int
BINLOG_CACHE_SIZE_GREATER_THAN_MAX: int
WARN_INDEX_NOT_APPLICABLE: int
PARTITION_EXCHANGE_FOREIGN_KEY: int
RPL_INFO_DATA_TOO_LONG: int
BINLOG_STMT_CACHE_SIZE_GREATER_THAN_MAX: int
CANT_UPDATE_TABLE_IN_CREATE_TABLE_SELECT: int
PARTITION_CLAUSE_ON_NONPARTITIONED: int
ROW_DOES_NOT_MATCH_GIVEN_PARTITION_SET: int
CHANGE_RPL_INFO_REPOSITORY_FAILURE: int
WARNING_NOT_COMPLETE_ROLLBACK_WITH_CREATED_TEMP_TABLE: int
WARNING_NOT_COMPLETE_ROLLBACK_WITH_DROPPED_TEMP_TABLE: int
MTS_FEATURE_IS_NOT_SUPPORTED: int
MTS_UPDATED_DBS_GREATER_MAX: int
MTS_CANT_PARALLEL: int
MTS_INCONSISTENT_DATA: int
FULLTEXT_NOT_SUPPORTED_WITH_PARTITIONING: int
DA_INVALID_CONDITION_NUMBER: int
INSECURE_PLAIN_TEXT: int
INSECURE_CHANGE_MASTER: int
FOREIGN_DUPLICATE_KEY_WITH_CHILD_INFO: int
FOREIGN_DUPLICATE_KEY_WITHOUT_CHILD_INFO: int
SQLTHREAD_WITH_SECURE_SLAVE: int
TABLE_HAS_NO_FT: int
VARIABLE_NOT_SETTABLE_IN_SF_OR_TRIGGER: int
VARIABLE_NOT_SETTABLE_IN_TRANSACTION: int
SET_STATEMENT_CANNOT_INVOKE_FUNCTION: int
GTID_NEXT_CANT_BE_AUTOMATIC_IF_GTID_NEXT_LIST_IS_NON_NULL: int
MALFORMED_GTID_SET_SPECIFICATION: int
MALFORMED_GTID_SET_ENCODING: int
MALFORMED_GTID_SPECIFICATION: int
GNO_EXHAUSTED: int
BAD_SLAVE_AUTO_POSITION: int
AUTO_POSITION_REQUIRES_GTID_MODE_NOT_OFF: int
CANT_DO_IMPLICIT_COMMIT_IN_TRX_WHEN_GTID_NEXT_IS_SET: int
GTID_MODE_ON_REQUIRES_ENFORCE_GTID_CONSISTENCY_ON: int
CANT_SET_GTID_NEXT_TO_GTID_WHEN_GTID_MODE_IS_OFF: int
CANT_SET_GTID_NEXT_TO_ANONYMOUS_WHEN_GTID_MODE_IS_ON: int
CANT_SET_GTID_NEXT_LIST_TO_NON_NULL_WHEN_GTID_MODE_IS_OFF: int
GTID_UNSAFE_NON_TRANSACTIONAL_TABLE: int
GTID_UNSAFE_CREATE_SELECT: int
GTID_UNSAFE_CREATE_DROP_TEMPORARY_TABLE_IN_TRANSACTION: int
GTID_MODE_CAN_ONLY_CHANGE_ONE_STEP_AT_A_TIME: int
MASTER_HAS_PURGED_REQUIRED_GTIDS: int
CANT_SET_GTID_NEXT_WHEN_OWNING_GTID: int
UNKNOWN_EXPLAIN_FORMAT: int
CANT_EXECUTE_IN_READ_ONLY_TRANSACTION: int
TOO_LONG_TABLE_PARTITION_COMMENT: int
SLAVE_CONFIGURATION: int
INNODB_FT_LIMIT: int
INNODB_NO_FT_TEMP_TABLE: int
INNODB_FT_WRONG_DOCID_COLUMN: int
INNODB_FT_WRONG_DOCID_INDEX: int
INNODB_ONLINE_LOG_TOO_BIG: int
UNKNOWN_ALTER_ALGORITHM: int
UNKNOWN_ALTER_LOCK: int
MTS_CHANGE_MASTER_CANT_RUN_WITH_GAPS: int
MTS_RECOVERY_FAILURE: int
MTS_RESET_WORKERS: int
COL_COUNT_DOESNT_MATCH_CORRUPTED_V2: int
SLAVE_SILENT_RETRY_TRANSACTION: int
DISCARD_FK_CHECKS_RUNNING: int
TABLE_SCHEMA_MISMATCH: int
TABLE_IN_SYSTEM_TABLESPACE: int
IO_READ_ERROR: int
IO_WRITE_ERROR: int
TABLESPACE_MISSING: int
TABLESPACE_EXISTS: int
TABLESPACE_DISCARDED: int
INTERNAL_ERROR: int
INNODB_IMPORT_ERROR: int
INNODB_INDEX_CORRUPT: int
INVALID_YEAR_COLUMN_LENGTH: int
NOT_VALID_PASSWORD: int
MUST_CHANGE_PASSWORD: int
FK_NO_INDEX_CHILD: int
FK_NO_INDEX_PARENT: int
FK_FAIL_ADD_SYSTEM: int
FK_CANNOT_OPEN_PARENT: int
FK_INCORRECT_OPTION: int
FK_DUP_NAME: int
PASSWORD_FORMAT: int
FK_COLUMN_CANNOT_DROP: int
FK_COLUMN_CANNOT_DROP_CHILD: int
FK_COLUMN_NOT_NULL: int
DUP_INDEX: int
FK_COLUMN_CANNOT_CHANGE: int
FK_COLUMN_CANNOT_CHANGE_CHILD: int
MALFORMED_PACKET: int
READ_ONLY_MODE: int
GTID_NEXT_TYPE_UNDEFINED_GTID: int
VARIABLE_NOT_SETTABLE_IN_SP: int
CANT_SET_GTID_PURGED_WHEN_GTID_EXECUTED_IS_NOT_EMPTY: int
CANT_SET_GTID_PURGED_WHEN_OWNED_GTIDS_IS_NOT_EMPTY: int
GTID_PURGED_WAS_CHANGED: int
GTID_EXECUTED_WAS_CHANGED: int
BINLOG_STMT_MODE_AND_NO_REPL_TABLES: int
ALTER_OPERATION_NOT_SUPPORTED: int
ALTER_OPERATION_NOT_SUPPORTED_REASON: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_COPY: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_PARTITION: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_FK_RENAME: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_COLUMN_TYPE: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_FK_CHECK: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_NOPK: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_AUTOINC: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_HIDDEN_FTS: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_CHANGE_FTS: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_FTS: int
SQL_SLAVE_SKIP_COUNTER_NOT_SETTABLE_IN_GTID_MODE: int
DUP_UNKNOWN_IN_INDEX: int
IDENT_CAUSES_TOO_LONG_PATH: int
ALTER_OPERATION_NOT_SUPPORTED_REASON_NOT_NULL: int
MUST_CHANGE_PASSWORD_LOGIN: int
ROW_IN_WRONG_PARTITION: int
MTS_EVENT_BIGGER_PENDING_JOBS_SIZE_MAX: int
BINLOG_LOGICAL_CORRUPTION: int
WARN_PURGE_LOG_IN_USE: int
WARN_PURGE_LOG_IS_ACTIVE: int
AUTO_INCREMENT_CONFLICT: int
WARN_ON_BLOCKHOLE_IN_RBR: int
SLAVE_MI_INIT_REPOSITORY: int
SLAVE_RLI_INIT_REPOSITORY: int
ACCESS_DENIED_CHANGE_USER_ERROR: int
INNODB_READ_ONLY: int
STOP_SLAVE_SQL_THREAD_TIMEOUT: int
STOP_SLAVE_IO_THREAD_TIMEOUT: int
TABLE_CORRUPT: int
TEMP_FILE_WRITE_FAILURE: int
INNODB_FT_AUX_NOT_HEX_ID: int
OLD_TEMPORALS_UPGRADED: int
INNODB_FORCED_RECOVERY: int
AES_INVALID_IV: int
PLUGIN_CANNOT_BE_UNINSTALLED: int
GTID_UNSAFE_BINLOG_SPLITTABLE_STATEMENT_AND_ASSIGNED_GTID: int
SLAVE_HAS_MORE_GTIDS_THAN_MASTER: int
MISSING_KEY: int
ERROR_LAST: int



from collections.abc import Iterable

from consolemenu import ConsoleMenu as ConsoleMenu
from consolemenu.items import SelectionItem as SelectionItem
from consolemenu.menu_formatter import MenuFormatBuilder
from consolemenu.screen import Screen

class SelectionMenu(ConsoleMenu):
    def __init__(
        self,
        strings: Iterable[str],
        title: str | None = ...,
        subtitle: str | None = ...,
        screen: Screen | None = ...,
        formatter: MenuFormatBuilder | None = ...,
        prologue_text: str | None = ...,
        epilogue_text: str | None = ...,
        show_exit_option: bool = ...,
        exit_option_text: str = ...,
        clear_screen: bool = ...,
    ) -> None: ...
    @classmethod
    def get_selection(
        cls,
        strings: Iterable[str],
        title: str = ...,
        subtitle: str | None = ...,
        show_exit_option: bool = ...,
        _menu: ConsoleMenu | None = ...,
    ) -> int: ...
    def append_string(self, string: str) -> None: ...



from collections.abc import Container, Iterable

from .css_sanitizer import CSSSanitizer
from .linkifier import DEFAULT_CALLBACKS as DEFAULT_CALLBACKS, Linker as Linker, _Callback
from .sanitizer import (
    ALLOWED_ATTRIBUTES as ALLOWED_ATTRIBUTES,
    ALLOWED_PROTOCOLS as ALLOWED_PROTOCOLS,
    ALLOWED_TAGS as ALLOWED_TAGS,
    Cleaner as Cleaner,
    _Attributes,
)

__all__ = ["clean", "linkify"]

__releasedate__: str
__version__: str

def clean(
    text: str,
    tags: Container[str] = ...,
    attributes: _Attributes = ...,
    protocols: Container[str] = ...,
    strip: bool = ...,
    strip_comments: bool = ...,
    css_sanitizer: CSSSanitizer | None = ...,
) -> str: ...
def linkify(
    text: str, callbacks: Iterable[_Callback] = ..., skip_tags: Container[str] | None = ..., parse_email: bool = ...
) -> str: ...



from re import Pattern

from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

def get_attrs(str): ...
def isheader(elem): ...

class AttrListTreeprocessor(Treeprocessor):
    BASE_RE: str
    HEADER_RE: Pattern[str]
    BLOCK_RE: Pattern[str]
    INLINE_RE: Pattern[str]
    NAME_RE: Pattern[str]
    def assign_attrs(self, elem, attrs) -> None: ...
    def sanitize_name(self, name): ...

class AttrListExtension(Extension): ...

def makeExtension(**kwargs): ...



from typing import Any

class _UnpickleDispatch:
    def __call__(self, _instance_cls): ...

class _Dispatch:
    def __init__(self, parent, instance_cls: Any | None = ...) -> None: ...
    def __getattr__(self, name): ...
    def __reduce__(self): ...

class _EventMeta(type):
    def __init__(cls, classname, bases, dict_) -> None: ...

class Events:
    dispatch: Any

class _JoinedDispatcher:
    local: Any
    parent: Any
    def __init__(self, local, parent) -> None: ...
    def __getattr__(self, name): ...

class dispatcher:
    dispatch: Any
    events: Any
    def __init__(self, events) -> None: ...
    def __get__(self, obj, cls): ...

class slots_dispatcher(dispatcher):
    def __get__(self, obj, cls): ...



from _typeshed import Self
from typing import Any

from ..span import Span
from ..tracer import Tracer
from .context import SpanContext
from .tracer import MockTracer

class MockSpan(Span):
    operation_name: str | None
    start_time: Any
    parent_id: int | None
    tags: dict[str, Any]
    finish_time: float
    finished: bool
    logs: list[LogData]
    def __init__(
        self,
        tracer: Tracer,
        operation_name: str | None = ...,
        context: SpanContext | None = ...,
        parent_id: int | None = ...,
        tags: dict[str, Any] | None = ...,
        start_time: float | None = ...,
    ) -> None: ...
    @property
    def tracer(self) -> MockTracer: ...
    @property
    def context(self) -> SpanContext: ...
    def set_operation_name(self: Self, operation_name: str) -> Self: ...
    def set_tag(self: Self, key: str, value: str | bool | float) -> Self: ...
    def log_kv(self: Self, key_values: dict[str, Any], timestamp: float | None = ...) -> Self: ...
    def set_baggage_item(self: Self, key: str, value: str) -> Self: ...

class LogData:
    key_values: dict[str, Any]
    timestamp: float | None
    def __init__(self, key_values: dict[str, Any], timestamp: float | None = ...) -> None: ...



from typing import Any

from braintree.configuration import Configuration as Configuration
from braintree.resource import Resource as Resource

class AccountUpdaterDailyReport(Resource):
    report_url: Any
    report_date: Any
    def __init__(self, gateway, attributes) -> None: ...



import sys
from _typeshed import SupportsWrite
from collections.abc import Callable, Sequence
from re import Pattern
from types import TracebackType
from typing import Any, TextIO
from typing_extensions import TypeAlias

if sys.platform == "win32":
    from .winterm import WinTerm

    winterm: WinTerm
else:
    winterm: None

class StreamWrapper:
    def __init__(self, wrapped: TextIO, converter: SupportsWrite[str]) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __enter__(self, *args: object, **kwargs: object) -> TextIO: ...
    def __exit__(
        self, __t: type[BaseException] | None, __value: BaseException | None, __traceback: TracebackType | None, **kwargs: Any
    ) -> None: ...
    def write(self, text: str) -> None: ...
    def isatty(self) -> bool: ...
    @property
    def closed(self) -> bool: ...

_WinTermCall: TypeAlias = Callable[[int | None, bool, bool], None]
_WinTermCallDict: TypeAlias = dict[int, tuple[_WinTermCall] | tuple[_WinTermCall, int] | tuple[_WinTermCall, int, bool]]

class AnsiToWin32:
    ANSI_CSI_RE: Pattern[str] = ...
    ANSI_OSC_RE: Pattern[str] = ...
    wrapped: TextIO = ...
    autoreset: bool = ...
    stream: StreamWrapper = ...
    strip: bool = ...
    convert: bool = ...
    win32_calls: _WinTermCallDict = ...
    on_stderr: bool = ...
    def __init__(self, wrapped: TextIO, convert: bool | None = ..., strip: bool | None = ..., autoreset: bool = ...) -> None: ...
    def should_wrap(self) -> bool: ...
    def get_win32_calls(self) -> _WinTermCallDict: ...
    def write(self, text: str) -> None: ...
    def reset_all(self) -> None: ...
    def write_and_convert(self, text: str) -> None: ...
    def write_plain_text(self, text: str, start: int, end: int) -> None: ...
    def convert_ansi(self, paramstring: str, command: str) -> None: ...
    def extract_params(self, command: str, paramstring: str) -> tuple[int, ...]: ...
    def call_win32(self, command: str, params: Sequence[int]) -> None: ...
    def convert_osc(self, text: str) -> str: ...
    def flush(self) -> None: ...



from _typeshed import Incomplete

from pythoncom import com_error as com_error

def RegisterInterfaces(typelibGUID, lcid, major, minor, interface_names: Incomplete | None = ...): ...

class Arg:
    name: Incomplete
    size: Incomplete
    offset: int
    def __init__(self, arg_info, name: Incomplete | None = ...) -> None: ...

class Method:
    dispid: Incomplete
    invkind: Incomplete
    name: Incomplete
    args: Incomplete
    cbArgs: Incomplete
    def __init__(self, method_info, isEventSink: int = ...) -> None: ...

class Definition:
    def __init__(self, iid, is_dispatch, method_defs) -> None: ...
    def iid(self): ...
    def vtbl_argsizes(self): ...
    def vtbl_argcounts(self): ...
    def dispatch(self, ob, index, argPtr, ReadFromInTuple=..., WriteFromOutTuple=...): ...



import decimal
from datetime import date
from re import Pattern
from typing_extensions import Literal

from babel.core import Locale

long = int
LC_NUMERIC: str | None

class UnknownCurrencyError(Exception):
    identifier: str
    def __init__(self, identifier: str) -> None: ...

def list_currencies(locale: Locale | str | None = ...) -> set[str]: ...
def validate_currency(currency: str, locale: Locale | str | None = ...) -> None: ...
def is_currency(currency: str, locale: Locale | str | None = ...) -> bool: ...
def normalize_currency(currency: str, locale: Locale | str | None = ...) -> str | None: ...
def get_currency_name(currency: str, count: float | decimal.Decimal | None = ..., locale: Locale | str | None = ...) -> str: ...
def get_currency_symbol(currency: str, locale: Locale | str | None = ...) -> str: ...
def get_currency_precision(currency: str) -> int: ...
def get_currency_unit_pattern(currency: str, count: float | None = ..., locale: Locale | str | None = ...) -> str: ...
def get_territory_currencies(
    territory: str,
    start_date: date | None = ...,
    end_date: date | None = ...,
    tender: bool = ...,
    non_tender: bool = ...,
    include_details: bool = ...,
) -> list[str]: ...
def get_decimal_symbol(locale: Locale | str | None = ...) -> str: ...
def get_plus_sign_symbol(locale: Locale | str | None = ...) -> str: ...
def get_minus_sign_symbol(locale: Locale | str | None = ...) -> str: ...
def get_exponential_symbol(locale: Locale | str | None = ...) -> str: ...
def get_group_symbol(locale: Locale | str | None = ...) -> str: ...
def format_number(number: float | decimal.Decimal, locale: Locale | str | None = ...) -> str: ...
def get_decimal_precision(number: decimal.Decimal) -> int: ...
def get_decimal_quantum(precision: int | decimal.Decimal) -> decimal.Decimal: ...
def format_decimal(
    number: float | decimal.Decimal,
    format: str | None = ...,
    locale: Locale | str | None = ...,
    decimal_quantization: bool = ...,
    group_separator: bool = ...,
): ...
def format_compact_decimal(
    number: float, *, format_type: Literal["short", "long"] = ..., locale: Locale | str | None = ..., fraction_digits: int = ...
) -> str: ...

class UnknownCurrencyFormatError(KeyError): ...

def format_currency(
    number: float | decimal.Decimal,
    currency: str,
    format: str | None = ...,
    locale: Locale | str | None = ...,
    currency_digits: bool = ...,
    format_type: Literal["name", "standard", "accounting"] = ...,
    decimal_quantization: bool = ...,
    group_separator: bool = ...,
) -> str: ...
def format_percent(
    number: float | decimal.Decimal,
    format: str | None = ...,
    locale: Locale | str | None = ...,
    decimal_quantization: bool = ...,
    group_separator: bool = ...,
) -> str: ...
def format_scientific(
    number: float | decimal.Decimal, format: str | None = ..., locale: Locale | str | None = ..., decimal_quantization: bool = ...
) -> str: ...

class NumberFormatError(ValueError):
    suggestions: str | None
    def __init__(self, message: str, suggestions: str | None = ...) -> None: ...

def parse_number(string: str, locale: Locale | str | None = ...) -> int: ...
def parse_decimal(string: str, locale: Locale | str | None = ..., strict: bool = ...) -> decimal.Decimal: ...

PREFIX_END: str
NUMBER_TOKEN: str
PREFIX_PATTERN: str
NUMBER_PATTERN: str
SUFFIX_PATTERN: str
number_re: Pattern[str]

def parse_grouping(p: str) -> tuple[int, int]: ...
def parse_pattern(pattern: NumberPattern | str) -> NumberPattern: ...

class NumberPattern:
    pattern: str
    prefix: tuple[str, str]
    suffix: tuple[str, str]
    grouping: tuple[int, int]
    int_prec: tuple[int, int]
    frac_prec: tuple[int, int]
    exp_prec: tuple[int, int] | None
    exp_plus: bool | None
    scale: Literal[0, 2, 3]
    def __init__(
        self,
        pattern: str,
        prefix: tuple[str, str],
        suffix: tuple[str, str],
        grouping: tuple[int, int],
        int_prec: tuple[int, int],
        frac_prec: tuple[int, int],
        exp_prec: tuple[int, int] | None,
        exp_plus: bool | None,
    ) -> None: ...
    def compute_scale(self) -> Literal[0, 2, 3]: ...
    def scientific_notation_elements(
        self, value: decimal.Decimal, locale: Locale | str | None
    ) -> tuple[decimal.Decimal, int, str]: ...
    def apply(
        self,
        value: float | decimal.Decimal,
        locale: Locale | str | None,
        currency: str | None = ...,
        currency_digits: bool = ...,
        decimal_quantization: bool = ...,
        force_frac: int | None = ...,
        group_separator: bool = ...,
    ) -> str: ...



__all__ = ["version", "bootstrap"]

def version() -> str: ...
def bootstrap(
    *,
    root: str | None = ...,
    upgrade: bool = ...,
    user: bool = ...,
    altinstall: bool = ...,
    default_pip: bool = ...,
    verbosity: int = ...,
) -> None: ...



from _typeshed import Incomplete
from typing import Any
from typing_extensions import TypeAlias

__all__ = ["TqdmCallback"]

_Callback: TypeAlias = Any  # Actually tensorflow.keras.callbacks.Callback

class TqdmCallback(_Callback):
    @staticmethod
    def bar2callback(bar, pop: Incomplete | None = ..., delta=...): ...
    tqdm_class: Incomplete
    epoch_bar: Incomplete
    on_epoch_end: Incomplete
    batches: Incomplete
    verbose: Incomplete
    batch_bar: Incomplete
    on_batch_end: Incomplete
    def __init__(
        self,
        epochs: Incomplete | None = ...,
        data_size: Incomplete | None = ...,
        batch_size: Incomplete | None = ...,
        verbose: int = ...,
        tqdm_class=...,
        **tqdm_kwargs,
    ) -> None: ...
    def on_train_begin(self, *_, **__) -> None: ...
    def on_epoch_begin(self, epoch, *_, **__) -> None: ...
    def on_train_end(self, *_, **__) -> None: ...
    def display(self) -> None: ...



from typing import Any

from .base import Client as Client

class BackendApplicationClient(Client):
    grant_type: str
    def prepare_request_body(self, body: str = ..., scope: Any | None = ..., include_client_id: bool = ..., **kwargs): ...  # type: ignore[override]



from typing import Any

UNQUOTE_PAIRS: Any
unquote: Any
tchar: Any
token: Any
token68: Any
quoted_string: Any
auth_param_name: Any
auth_param: Any
params: Any
scheme: Any
challenge: Any
authentication_info: Any
www_authenticate: Any



from typing import Any

from .. import util

class RefCollection(util.MemoizedSlots):
    ref: Any

class _empty_collection:
    def append(self, element) -> None: ...
    def extend(self, other) -> None: ...
    def remove(self, element) -> None: ...
    def __iter__(self): ...
    def clear(self) -> None: ...

class _ClsLevelDispatch(RefCollection):
    name: Any
    clsname: Any
    arg_names: Any
    has_kw: Any
    legacy_signatures: Any
    def __init__(self, parent_dispatch_cls, fn): ...
    def insert(self, event_key, propagate) -> None: ...
    def append(self, event_key, propagate) -> None: ...
    def update_subclass(self, target) -> None: ...
    def remove(self, event_key) -> None: ...
    def clear(self) -> None: ...
    def for_modify(self, obj): ...

class _InstanceLevelDispatch(RefCollection): ...

class _EmptyListener(_InstanceLevelDispatch):
    propagate: Any
    listeners: Any
    parent: Any
    parent_listeners: Any
    name: Any
    def __init__(self, parent, target_cls) -> None: ...
    def for_modify(self, obj): ...
    exec_once: Any
    exec_once_unless_exception: Any
    insert: Any
    append: Any
    remove: Any
    clear: Any
    def __call__(self, *args, **kw) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...
    def __bool__(self): ...
    __nonzero__: Any

class _CompoundListener(_InstanceLevelDispatch):
    def exec_once(self, *args, **kw) -> None: ...
    def exec_once_unless_exception(self, *args, **kw) -> None: ...
    def __call__(self, *args, **kw) -> None: ...
    def __len__(self): ...
    def __iter__(self): ...
    def __bool__(self): ...
    __nonzero__: Any

class _ListenerCollection(_CompoundListener):
    parent_listeners: Any
    parent: Any
    name: Any
    listeners: Any
    propagate: Any
    def __init__(self, parent, target_cls) -> None: ...
    def for_modify(self, obj): ...
    def insert(self, event_key, propagate) -> None: ...
    def append(self, event_key, propagate) -> None: ...
    def remove(self, event_key) -> None: ...
    def clear(self) -> None: ...

class _JoinedListener(_CompoundListener):
    parent: Any
    name: Any
    local: Any
    parent_listeners: Any
    def __init__(self, parent, name, local) -> None: ...
    @property
    def listeners(self): ...
    def for_modify(self, obj): ...
    def insert(self, event_key, propagate) -> None: ...
    def append(self, event_key, propagate) -> None: ...
    def remove(self, event_key) -> None: ...
    def clear(self) -> None: ...



import sys
from _typeshed import Self
from abc import abstractmethod
from time import struct_time
from typing import ClassVar, NamedTuple, NoReturn, TypeVar, overload
from typing_extensions import Literal, TypeAlias, final

if sys.version_info >= (3, 11):
    __all__ = ("date", "datetime", "time", "timedelta", "timezone", "tzinfo", "MINYEAR", "MAXYEAR", "UTC")
elif sys.version_info >= (3, 9):
    __all__ = ("date", "datetime", "time", "timedelta", "timezone", "tzinfo", "MINYEAR", "MAXYEAR")

_D = TypeVar("_D", bound=date)

MINYEAR: Literal[1]
MAXYEAR: Literal[9999]

class tzinfo:
    @abstractmethod
    def tzname(self, __dt: datetime | None) -> str | None: ...
    @abstractmethod
    def utcoffset(self, __dt: datetime | None) -> timedelta | None: ...
    @abstractmethod
    def dst(self, __dt: datetime | None) -> timedelta | None: ...
    def fromutc(self, __dt: datetime) -> datetime: ...

# Alias required to avoid name conflicts with date(time).tzinfo.
_TzInfo: TypeAlias = tzinfo

@final
class timezone(tzinfo):
    utc: ClassVar[timezone]
    min: ClassVar[timezone]
    max: ClassVar[timezone]
    def __init__(self, offset: timedelta, name: str = ...) -> None: ...
    def tzname(self, __dt: datetime | None) -> str: ...
    def utcoffset(self, __dt: datetime | None) -> timedelta: ...
    def dst(self, __dt: datetime | None) -> None: ...

if sys.version_info >= (3, 11):
    UTC: timezone

if sys.version_info >= (3, 9):
    class _IsoCalendarDate(NamedTuple):
        year: int
        week: int
        weekday: int

class date:
    min: ClassVar[date]
    max: ClassVar[date]
    resolution: ClassVar[timedelta]
    def __new__(cls: type[Self], year: int, month: int, day: int) -> Self: ...
    @classmethod
    def fromtimestamp(cls: type[Self], __timestamp: float) -> Self: ...
    @classmethod
    def today(cls: type[Self]) -> Self: ...
    @classmethod
    def fromordinal(cls: type[Self], __n: int) -> Self: ...
    @classmethod
    def fromisoformat(cls: type[Self], __date_string: str) -> Self: ...
    if sys.version_info >= (3, 8):
        @classmethod
        def fromisocalendar(cls: type[Self], year: int, week: int, day: int) -> Self: ...

    @property
    def year(self) -> int: ...
    @property
    def month(self) -> int: ...
    @property
    def day(self) -> int: ...
    def ctime(self) -> str: ...
    def strftime(self, __format: str) -> str: ...
    def __format__(self, __fmt: str) -> str: ...
    def isoformat(self) -> str: ...
    def timetuple(self) -> struct_time: ...
    def toordinal(self) -> int: ...
    def replace(self: Self, year: int = ..., month: int = ..., day: int = ...) -> Self: ...
    def __le__(self, __other: date) -> bool: ...
    def __lt__(self, __other: date) -> bool: ...
    def __ge__(self, __other: date) -> bool: ...
    def __gt__(self, __other: date) -> bool: ...
    if sys.version_info >= (3, 8):
        def __add__(self: Self, __other: timedelta) -> Self: ...
        def __radd__(self: Self, __other: timedelta) -> Self: ...
        @overload
        def __sub__(self: Self, __other: timedelta) -> Self: ...
        @overload
        def __sub__(self, __other: datetime) -> NoReturn: ...
        @overload
        def __sub__(self: _D, __other: _D) -> timedelta: ...
    else:
        # Prior to Python 3.8, arithmetic operations always returned `date`, even in subclasses
        def __add__(self, __other: timedelta) -> date: ...
        def __radd__(self, __other: timedelta) -> date: ...
        @overload
        def __sub__(self, __other: timedelta) -> date: ...
        @overload
        def __sub__(self, __other: datetime) -> NoReturn: ...
        @overload
        def __sub__(self, __other: date) -> timedelta: ...

    def weekday(self) -> int: ...
    def isoweekday(self) -> int: ...
    if sys.version_info >= (3, 9):
        def isocalendar(self) -> _IsoCalendarDate: ...
    else:
        def isocalendar(self) -> tuple[int, int, int]: ...

class time:
    min: ClassVar[time]
    max: ClassVar[time]
    resolution: ClassVar[timedelta]
    def __new__(
        cls: type[Self],
        hour: int = ...,
        minute: int = ...,
        second: int = ...,
        microsecond: int = ...,
        tzinfo: _TzInfo | None = ...,
        *,
        fold: int = ...,
    ) -> Self: ...
    @property
    def hour(self) -> int: ...
    @property
    def minute(self) -> int: ...
    @property
    def second(self) -> int: ...
    @property
    def microsecond(self) -> int: ...
    @property
    def tzinfo(self) -> _TzInfo | None: ...
    @property
    def fold(self) -> int: ...
    def __le__(self, __other: time) -> bool: ...
    def __lt__(self, __other: time) -> bool: ...
    def __ge__(self, __other: time) -> bool: ...
    def __gt__(self, __other: time) -> bool: ...
    def isoformat(self, timespec: str = ...) -> str: ...
    @classmethod
    def fromisoformat(cls: type[Self], __time_string: str) -> Self: ...
    def strftime(self, __format: str) -> str: ...
    def __format__(self, __fmt: str) -> str: ...
    def utcoffset(self) -> timedelta | None: ...
    def tzname(self) -> str | None: ...
    def dst(self) -> timedelta | None: ...
    def replace(
        self: Self,
        hour: int = ...,
        minute: int = ...,
        second: int = ...,
        microsecond: int = ...,
        tzinfo: _TzInfo | None = ...,
        *,
        fold: int = ...,
    ) -> Self: ...

_Date: TypeAlias = date
_Time: TypeAlias = time

class timedelta:
    min: ClassVar[timedelta]
    max: ClassVar[timedelta]
    resolution: ClassVar[timedelta]
    def __new__(
        cls: type[Self],
        days: float = ...,
        seconds: float = ...,
        microseconds: float = ...,
        milliseconds: float = ...,
        minutes: float = ...,
        hours: float = ...,
        weeks: float = ...,
    ) -> Self: ...
    @property
    def days(self) -> int: ...
    @property
    def seconds(self) -> int: ...
    @property
    def microseconds(self) -> int: ...
    def total_seconds(self) -> float: ...
    def __add__(self, __other: timedelta) -> timedelta: ...
    def __radd__(self, __other: timedelta) -> timedelta: ...
    def __sub__(self, __other: timedelta) -> timedelta: ...
    def __rsub__(self, __other: timedelta) -> timedelta: ...
    def __neg__(self) -> timedelta: ...
    def __pos__(self) -> timedelta: ...
    def __abs__(self) -> timedelta: ...
    def __mul__(self, __other: float) -> timedelta: ...
    def __rmul__(self, __other: float) -> timedelta: ...
    @overload
    def __floordiv__(self, __other: timedelta) -> int: ...
    @overload
    def __floordiv__(self, __other: int) -> timedelta: ...
    @overload
    def __truediv__(self, __other: timedelta) -> float: ...
    @overload
    def __truediv__(self, __other: float) -> timedelta: ...
    def __mod__(self, __other: timedelta) -> timedelta: ...
    def __divmod__(self, __other: timedelta) -> tuple[int, timedelta]: ...
    def __le__(self, __other: timedelta) -> bool: ...
    def __lt__(self, __other: timedelta) -> bool: ...
    def __ge__(self, __other: timedelta) -> bool: ...
    def __gt__(self, __other: timedelta) -> bool: ...
    def __bool__(self) -> bool: ...

class datetime(date):
    min: ClassVar[datetime]
    max: ClassVar[datetime]
    def __new__(
        cls: type[Self],
        year: int,
        month: int,
        day: int,
        hour: int = ...,
        minute: int = ...,
        second: int = ...,
        microsecond: int = ...,
        tzinfo: _TzInfo | None = ...,
        *,
        fold: int = ...,
    ) -> Self: ...
    @property
    def hour(self) -> int: ...
    @property
    def minute(self) -> int: ...
    @property
    def second(self) -> int: ...
    @property
    def microsecond(self) -> int: ...
    @property
    def tzinfo(self) -> _TzInfo | None: ...
    @property
    def fold(self) -> int: ...
    # The first parameter in `fromtimestamp` is actually positional-or-keyword,
    # but it is named "timestamp" in the C implementation and "t" in the Python implementation,
    # so it is only truly *safe* to pass it as a positional argument.
    @classmethod
    def fromtimestamp(cls: type[Self], __timestamp: float, tz: _TzInfo | None = ...) -> Self: ...
    @classmethod
    def utcfromtimestamp(cls: type[Self], __t: float) -> Self: ...
    if sys.version_info >= (3, 8):
        @classmethod
        def now(cls: type[Self], tz: _TzInfo | None = ...) -> Self: ...
    else:
        @overload
        @classmethod
        def now(cls: type[Self], tz: None = ...) -> Self: ...
        @overload
        @classmethod
        def now(cls, tz: _TzInfo) -> datetime: ...

    @classmethod
    def utcnow(cls: type[Self]) -> Self: ...
    @classmethod
    def combine(cls, date: _Date, time: _Time, tzinfo: _TzInfo | None = ...) -> datetime: ...
    def timestamp(self) -> float: ...
    def utctimetuple(self) -> struct_time: ...
    def date(self) -> _Date: ...
    def time(self) -> _Time: ...
    def timetz(self) -> _Time: ...
    def replace(
        self: Self,
        year: int = ...,
        month: int = ...,
        day: int = ...,
        hour: int = ...,
        minute: int = ...,
        second: int = ...,
        microsecond: int = ...,
        tzinfo: _TzInfo | None = ...,
        *,
        fold: int = ...,
    ) -> Self: ...
    if sys.version_info >= (3, 8):
        def astimezone(self: Self, tz: _TzInfo | None = ...) -> Self: ...
    else:
        def astimezone(self, tz: _TzInfo | None = ...) -> datetime: ...

    def isoformat(self, sep: str = ..., timespec: str = ...) -> str: ...
    @classmethod
    def strptime(cls, __date_string: str, __format: str) -> datetime: ...
    def utcoffset(self) -> timedelta | None: ...
    def tzname(self) -> str | None: ...
    def dst(self) -> timedelta | None: ...
    def __le__(self, __other: datetime) -> bool: ...  # type: ignore[override]
    def __lt__(self, __other: datetime) -> bool: ...  # type: ignore[override]
    def __ge__(self, __other: datetime) -> bool: ...  # type: ignore[override]
    def __gt__(self, __other: datetime) -> bool: ...  # type: ignore[override]
    if sys.version_info >= (3, 8):
        @overload  # type: ignore[override]
        def __sub__(self: Self, __other: timedelta) -> Self: ...
        @overload
        def __sub__(self: _D, __other: _D) -> timedelta: ...
    else:
        # Prior to Python 3.8, arithmetic operations always returned `datetime`, even in subclasses
        def __add__(self, __other: timedelta) -> datetime: ...
        def __radd__(self, __other: timedelta) -> datetime: ...
        @overload  # type: ignore[override]
        def __sub__(self, __other: datetime) -> timedelta: ...
        @overload
        def __sub__(self, __other: timedelta) -> datetime: ...



from typing import Any

from ...sql.dml import Insert as StandardInsert
from ...sql.elements import ClauseElement
from ...util import memoized_property

class Insert(StandardInsert):
    stringify_dialect: str
    inherit_cache: bool
    @memoized_property
    def excluded(self): ...
    def on_conflict_do_update(
        self, index_elements: Any | None = ..., index_where: Any | None = ..., set_: Any | None = ..., where: Any | None = ...
    ) -> None: ...
    def on_conflict_do_nothing(self, index_elements: Any | None = ..., index_where: Any | None = ...) -> None: ...

insert: Any

class OnConflictClause(ClauseElement):
    stringify_dialect: str
    constraint_target: Any
    inferred_target_elements: Any
    inferred_target_whereclause: Any
    def __init__(self, index_elements: Any | None = ..., index_where: Any | None = ...) -> None: ...

class OnConflictDoNothing(OnConflictClause):
    __visit_name__: str

class OnConflictDoUpdate(OnConflictClause):
    __visit_name__: str
    update_values_to_set: Any
    update_whereclause: Any
    def __init__(
        self, index_elements: Any | None = ..., index_where: Any | None = ..., set_: Any | None = ..., where: Any | None = ...
    ) -> None: ...



from typing import Any

from openpyxl.descriptors import Typed
from openpyxl.descriptors.serialisable import Serialisable

PRESET_COLORS: Any
SCHEME_COLORS: Any

class Transform(Serialisable): ...

class SystemColor(Serialisable):
    tagname: str
    namespace: Any
    tint: Any
    shade: Any
    comp: Any
    inv: Any
    gray: Any
    alpha: Any
    alphaOff: Any
    alphaMod: Any
    hue: Any
    hueOff: Any
    hueMod: Any
    sat: Any
    satOff: Any
    satMod: Any
    lum: Any
    lumOff: Any
    lumMod: Any
    red: Any
    redOff: Any
    redMod: Any
    green: Any
    greenOff: Any
    greenMod: Any
    blue: Any
    blueOff: Any
    blueMod: Any
    gamma: Any
    invGamma: Any
    val: Any
    lastClr: Any
    __elements__: Any
    def __init__(
        self,
        val: str = ...,
        lastClr: Any | None = ...,
        tint: Any | None = ...,
        shade: Any | None = ...,
        comp: Any | None = ...,
        inv: Any | None = ...,
        gray: Any | None = ...,
        alpha: Any | None = ...,
        alphaOff: Any | None = ...,
        alphaMod: Any | None = ...,
        hue: Any | None = ...,
        hueOff: Any | None = ...,
        hueMod: Any | None = ...,
        sat: Any | None = ...,
        satOff: Any | None = ...,
        satMod: Any | None = ...,
        lum: Any | None = ...,
        lumOff: Any | None = ...,
        lumMod: Any | None = ...,
        red: Any | None = ...,
        redOff: Any | None = ...,
        redMod: Any | None = ...,
        green: Any | None = ...,
        greenOff: Any | None = ...,
        greenMod: Any | None = ...,
        blue: Any | None = ...,
        blueOff: Any | None = ...,
        blueMod: Any | None = ...,
        gamma: Any | None = ...,
        invGamma: Any | None = ...,
    ) -> None: ...

class HSLColor(Serialisable):
    tagname: str
    hue: Any
    sat: Any
    lum: Any
    def __init__(self, hue: Any | None = ..., sat: Any | None = ..., lum: Any | None = ...) -> None: ...

class RGBPercent(Serialisable):
    tagname: str
    r: Any
    g: Any
    b: Any
    def __init__(self, r: Any | None = ..., g: Any | None = ..., b: Any | None = ...) -> None: ...

class SchemeColor(Serialisable):
    tagname: str
    namespace: Any
    tint: Any
    shade: Any
    comp: Any
    inv: Any
    gray: Any
    alpha: Any
    alphaOff: Any
    alphaMod: Any
    hue: Any
    hueOff: Any
    hueMod: Any
    sat: Any
    satOff: Any
    satMod: Any
    lum: Any
    lumOff: Any
    lumMod: Any
    red: Any
    redOff: Any
    redMod: Any
    green: Any
    greenOff: Any
    greenMod: Any
    blue: Any
    blueOff: Any
    blueMod: Any
    gamma: Any
    invGamma: Any
    val: Any
    __elements__: Any
    def __init__(
        self,
        tint: Any | None = ...,
        shade: Any | None = ...,
        comp: Any | None = ...,
        inv: Any | None = ...,
        gray: Any | None = ...,
        alpha: Any | None = ...,
        alphaOff: Any | None = ...,
        alphaMod: Any | None = ...,
        hue: Any | None = ...,
        hueOff: Any | None = ...,
        hueMod: Any | None = ...,
        sat: Any | None = ...,
        satOff: Any | None = ...,
        satMod: Any | None = ...,
        lum: Any | None = ...,
        lumOff: Any | None = ...,
        lumMod: Any | None = ...,
        red: Any | None = ...,
        redOff: Any | None = ...,
        redMod: Any | None = ...,
        green: Any | None = ...,
        greenOff: Any | None = ...,
        greenMod: Any | None = ...,
        blue: Any | None = ...,
        blueOff: Any | None = ...,
        blueMod: Any | None = ...,
        gamma: Any | None = ...,
        invGamma: Any | None = ...,
        val: Any | None = ...,
    ) -> None: ...

class ColorChoice(Serialisable):
    tagname: str
    namespace: Any
    scrgbClr: Any
    RGBPercent: Any
    srgbClr: Any
    RGB: Any
    hslClr: Any
    sysClr: Any
    schemeClr: Any
    prstClr: Any
    __elements__: Any
    def __init__(
        self,
        scrgbClr: Any | None = ...,
        srgbClr: Any | None = ...,
        hslClr: Any | None = ...,
        sysClr: Any | None = ...,
        schemeClr: Any | None = ...,
        prstClr: Any | None = ...,
    ) -> None: ...

class ColorMapping(Serialisable):
    tagname: str
    bg1: Any
    tx1: Any
    bg2: Any
    tx2: Any
    accent1: Any
    accent2: Any
    accent3: Any
    accent4: Any
    accent5: Any
    accent6: Any
    hlink: Any
    folHlink: Any
    extLst: Any
    def __init__(
        self,
        bg1: str = ...,
        tx1: str = ...,
        bg2: str = ...,
        tx2: str = ...,
        accent1: str = ...,
        accent2: str = ...,
        accent3: str = ...,
        accent4: str = ...,
        accent5: str = ...,
        accent6: str = ...,
        hlink: str = ...,
        folHlink: str = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class ColorChoiceDescriptor(Typed):
    expected_type: Any
    allow_none: bool
    def __set__(self, instance, value) -> None: ...



import threading
from collections.abc import Callable
from contextlib import AbstractContextManager
from multiprocessing.context import BaseContext
from types import TracebackType
from typing_extensions import TypeAlias

__all__ = ["Lock", "RLock", "Semaphore", "BoundedSemaphore", "Condition", "Event"]

_LockLike: TypeAlias = Lock | RLock

class Barrier(threading.Barrier):
    def __init__(
        self, parties: int, action: Callable[[], object] | None = ..., timeout: float | None = ..., *ctx: BaseContext
    ) -> None: ...

class BoundedSemaphore(Semaphore):
    def __init__(self, value: int = ..., *, ctx: BaseContext) -> None: ...

class Condition(AbstractContextManager[bool]):
    def __init__(self, lock: _LockLike | None = ..., *, ctx: BaseContext) -> None: ...
    def notify(self, n: int = ...) -> None: ...
    def notify_all(self) -> None: ...
    def wait(self, timeout: float | None = ...) -> bool: ...
    def wait_for(self, predicate: Callable[[], bool], timeout: float | None = ...) -> bool: ...
    def acquire(self, block: bool = ..., timeout: float | None = ...) -> bool: ...
    def release(self) -> None: ...
    def __exit__(
        self, __exc_type: type[BaseException] | None, __exc_val: BaseException | None, __exc_tb: TracebackType | None
    ) -> None: ...

class Event:
    def __init__(self, lock: _LockLike | None = ..., *, ctx: BaseContext) -> None: ...
    def is_set(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    def wait(self, timeout: float | None = ...) -> bool: ...

class Lock(SemLock):
    def __init__(self, *, ctx: BaseContext) -> None: ...

class RLock(SemLock):
    def __init__(self, *, ctx: BaseContext) -> None: ...

class Semaphore(SemLock):
    def __init__(self, value: int = ..., *, ctx: BaseContext) -> None: ...

# Not part of public API
class SemLock(AbstractContextManager[bool]):
    def acquire(self, block: bool = ..., timeout: float | None = ...) -> bool: ...
    def release(self) -> None: ...
    def __exit__(
        self, __exc_type: type[BaseException] | None, __exc_val: BaseException | None, __exc_tb: TracebackType | None
    ) -> None: ...



from typing import Any, ClassVar
from typing_extensions import Literal

from .TiffImagePlugin import TiffImageFile

class MicImageFile(TiffImageFile):
    format: ClassVar[Literal["MIC"]]
    fp: Any
    frame: Any
    def seek(self, frame) -> None: ...
    def tell(self): ...



from _typeshed import Incomplete

from win32.lib.pywintypes import error as error

def NetGetJoinInformation() -> tuple[str, Incomplete]: ...
def NetGroupGetInfo(server: str, groupname: str, level): ...
def NetGroupGetUsers(
    server: str, groupName: str, level, resumeHandle: int = ..., prefLen: int = ...
) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetGroupSetUsers(server: str, group: str, level, members: tuple[Incomplete, Incomplete]) -> None: ...
def NetGroupSetInfo(server: str, groupname: str, level, data) -> None: ...
def NetGroupAdd(server: str, level, data) -> None: ...
def NetGroupAddUser(server: str, group: str, username: str) -> None: ...
def NetGroupDel(server: str, groupname: str) -> None: ...
def NetGroupDelUser(server: str, group: str, username: str) -> None: ...
def NetGroupEnum(server: str, level, prefLen, resumeHandle=...) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetLocalGroupAddMembers(server: str, group: str, level, members: tuple[Incomplete, Incomplete]) -> None: ...
def NetLocalGroupDelMembers(server: str, group: str, members: list[str]) -> None: ...
def NetLocalGroupGetMembers(
    server: str, groupName: str, level, resumeHandle: int = ..., prefLen: int = ...
) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetLocalGroupSetMembers(server: str, group: str, level, members: tuple[Incomplete, Incomplete]) -> None: ...
def NetMessageBufferSend(domain: str, userName: str, fromName: str, message: str) -> None: ...
def NetMessageNameAdd(server, msgname) -> None: ...
def NetMessageNameDel(server, msgname) -> None: ...
def NetMessageNameEnum(Server) -> None: ...
def NetServerEnum(
    server: str, level, _type, prefLen, domain: str | None = ..., resumeHandle: int = ...
) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetServerGetInfo(server: str, level): ...
def NetServerSetInfo(server: str, level, data) -> None: ...
def NetShareAdd(server: str, level, data) -> None: ...
def NetShareDel(server: str, shareName: str, reserved: int = ...) -> None: ...
def NetShareCheck(server: str, deviceName: str) -> tuple[Incomplete, Incomplete]: ...
def NetShareEnum(
    server: str, level, prefLen, serverName, resumeHandle=...
) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetShareGetInfo(server: str, netname: str, level): ...
def NetShareSetInfo(server: str, netname: str, level, data) -> None: ...
def NetUserAdd(server: str, level, data) -> None: ...
def NetUserChangePassword(server: str, username: str, oldPassword: str, newPassword: str) -> None: ...
def NetUserEnum(server: str, level, arg, prefLen, resumeHandle=...) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetUserGetGroups(serverName: str, userName: str) -> list[tuple[Incomplete, Incomplete]]: ...
def NetUserGetInfo(server: str, username: str, level): ...
def NetUserGetLocalGroups(serverName: str, userName: str, flags) -> list[Incomplete]: ...
def NetUserSetInfo(server: str, username: str, level, data) -> None: ...
def NetUserDel(server: str, username: str) -> None: ...
def NetUserModalsGet(server: str, level): ...
def NetUserModalsSet(server: str, level, data) -> None: ...
def NetWkstaUserEnum(server: str, level, prefLen, resumeHandle=...) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetWkstaGetInfo(server: str, level): ...
def NetWkstaSetInfo(server: str, level, data) -> None: ...
def NetWkstaTransportEnum(
    server: str, level, prefLen, resumeHandle=...
) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetWkstaTransportAdd(server: str, level, data) -> None: ...
def NetWkstaTransportDel(server: str, TransportName: str, ucond: int = ...) -> None: ...
def NetServerDiskEnum(server: str, level): ...
def NetUseAdd(server: str, level, data) -> None: ...
def NetUseDel(server: str, useName: str, forceCond: int = ...) -> None: ...
def NetUseEnum(server: str, level, prefLen, resumeHandle=...) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def NetUseGetInfo(server: str, usename: str, level: int = ...): ...
def NetGetAnyDCName(server: str | None = ..., domain: str | None = ...) -> str: ...
def NetGetDCName(server: str | None = ..., domain: str | None = ...) -> str: ...
def NetSessionEnum(
    level, server: str | None = ..., client: str | None = ..., username: str | None = ...
) -> tuple[Incomplete, ...]: ...
def NetSessionDel(server: str, client: str | None = ..., username: str | None = ...) -> None: ...
def NetSessionGetInfo(level, server: str, client: str, username: str): ...
def NetFileEnum(
    level, servername: str | None = ..., basepath: str | None = ..., username: str | None = ...
) -> tuple[Incomplete, ...]: ...
def NetFileClose(servername: str, fileid) -> None: ...
def NetFileGetInfo(level, servername: str, fileid): ...
def NetStatisticsGet(server: str, service: str, level, options): ...
def NetServerComputerNameAdd(ServerName: str, EmulatedDomainName: str, EmulatedServerName: str) -> None: ...
def NetServerComputerNameDel(ServerName: str, EmulatedServerName: str) -> None: ...
def NetValidateName(Server: str, Name: str, NameType, Account: str | None = ..., Password: str | None = ...) -> None: ...
def NetValidatePasswordPolicy(Server: str, Qualifier, ValidationType, arg) -> None: ...
def NetLocalGroupAdd(*args, **kwargs): ...  # incomplete
def NetLocalGroupDel(*args, **kwargs): ...  # incomplete
def NetLocalGroupEnum(*args, **kwargs): ...  # incomplete
def NetLocalGroupGetInfo(*args, **kwargs): ...  # incomplete
def NetLocalGroupSetInfo(*args, **kwargs): ...  # incomplete

SERVICE_SERVER: str
SERVICE_WORKSTATION: str
USE_FORCE: int
USE_LOTS_OF_FORCE: int
USE_NOFORCE: int



from .menu_borders import (
    AsciiBorderStyle as AsciiBorderStyle,
    DoubleLineBorderStyle as DoubleLineBorderStyle,
    DoubleLineOuterLightInnerBorderStyle as DoubleLineOuterLightInnerBorderStyle,
    HeavyBorderStyle as HeavyBorderStyle,
    HeavyOuterLightInnerBorderStyle as HeavyOuterLightInnerBorderStyle,
    LightBorderStyle as LightBorderStyle,
    MenuBorderStyle as MenuBorderStyle,
    MenuBorderStyleFactory as MenuBorderStyleFactory,
    MenuBorderStyleType as MenuBorderStyleType,
)
from .menu_margins import MenuMargins as MenuMargins
from .menu_padding import MenuPadding as MenuPadding
from .menu_style import MenuStyle as MenuStyle



from _collections_abc import Generator, dict_keys
from _typeshed import Self
from types import TracebackType
from typing import Any
from typing_extensions import Literal, TypeAlias

from .pooling import ServerPool
from .server import Server

SASL_AVAILABLE_MECHANISMS: Any
CLIENT_STRATEGIES: Any

_ServerSequence: TypeAlias = (
    set[Server] | list[Server] | tuple[Server, ...] | Generator[Server, None, None] | dict_keys[Server, Any]
)

class Connection:
    connection_lock: Any
    last_error: str
    strategy_type: Any
    user: Any
    password: Any
    authentication: Any
    version: Any
    auto_referrals: Any
    request: Any
    response: Any | None
    result: Any
    bound: bool
    listening: bool
    closed: bool
    auto_bind: Any
    sasl_mechanism: Any
    sasl_credentials: Any
    socket: Any
    tls_started: bool
    sasl_in_progress: bool
    read_only: Any
    lazy: Any
    pool_name: Any
    pool_size: int | None
    cred_store: Any
    pool_lifetime: Any
    pool_keepalive: Any
    starting_tls: bool
    check_names: Any
    raise_exceptions: Any
    auto_range: Any
    extend: Any
    fast_decoder: Any
    receive_timeout: Any
    empty_attributes: Any
    use_referral_cache: Any
    auto_escape: Any
    auto_encode: Any
    source_address: Any
    source_port_list: Any
    server_pool: Any | None
    server: Any
    strategy: Any
    send: Any
    open: Any
    get_response: Any
    post_send_single_response: Any
    post_send_search: Any
    def __init__(
        self,
        server: Server | str | _ServerSequence | ServerPool,
        user: str | None = ...,
        password: str | None = ...,
        auto_bind: Literal["DEFAULT", "NONE", "NO_TLS", "TLS_BEFORE_BIND", "TLS_AFTER_BIND"] = ...,
        version: int = ...,
        authentication: Literal["ANONYMOUS", "SIMPLE", "SASL", "NTLM"] | None = ...,
        client_strategy: Literal[
            "SYNC",
            "SAFE_RESTARTABLE",
            "SAFE_SYNC",
            "ASYNC",
            "LDIF",
            "RESTARTABLE",
            "REUSABLE",
            "MOCK_SYNC",
            "MOCK_ASYNC",
            "ASYNC_STREAM",
        ] = ...,
        auto_referrals: bool = ...,
        auto_range: bool = ...,
        sasl_mechanism: str | None = ...,
        sasl_credentials: Any | None = ...,
        check_names: bool = ...,
        collect_usage: bool = ...,
        read_only: bool = ...,
        lazy: bool = ...,
        raise_exceptions: bool = ...,
        pool_name: str | None = ...,
        pool_size: int | None = ...,
        pool_lifetime: int | None = ...,
        cred_store: Any | None = ...,
        fast_decoder: bool = ...,
        receive_timeout: Any | None = ...,
        return_empty_attributes: bool = ...,
        use_referral_cache: bool = ...,
        auto_escape: bool = ...,
        auto_encode: bool = ...,
        pool_keepalive: Any | None = ...,
        source_address: str | None = ...,
        source_port: int | None = ...,
        source_port_list: Any | None = ...,
    ) -> None: ...
    def repr_with_sensitive_data_stripped(self): ...
    @property
    def stream(self): ...
    @stream.setter
    def stream(self, value) -> None: ...
    @property
    def usage(self): ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> Literal[False] | None: ...
    def bind(self, read_server_info: bool = ..., controls: Any | None = ...): ...
    def rebind(
        self,
        user: Any | None = ...,
        password: Any | None = ...,
        authentication: Any | None = ...,
        sasl_mechanism: Any | None = ...,
        sasl_credentials: Any | None = ...,
        read_server_info: bool = ...,
        controls: Any | None = ...,
    ): ...
    def unbind(self, controls: Any | None = ...): ...
    def search(
        self,
        search_base: str,
        search_filter: str,
        search_scope: Literal["BASE", "LEVEL", "SUBTREE"] = ...,
        dereference_aliases: Literal["NEVER", "SEARCH", "FINDING_BASE", "ALWAYS"] = ...,
        attributes: Any | None = ...,
        size_limit: int = ...,
        time_limit: int = ...,
        types_only: bool = ...,
        get_operational_attributes: bool = ...,
        controls: Any | None = ...,
        paged_size: int | None = ...,
        paged_criticality: bool = ...,
        paged_cookie: str | bytes | None = ...,
        auto_escape: bool | None = ...,
    ): ...
    def compare(self, dn, attribute, value, controls: Any | None = ...): ...
    def add(self, dn, object_class: Any | None = ..., attributes: Any | None = ..., controls: Any | None = ...): ...
    def delete(self, dn, controls: Any | None = ...): ...
    def modify(self, dn, changes, controls: Any | None = ...): ...
    def modify_dn(
        self, dn, relative_dn, delete_old_dn: bool = ..., new_superior: Any | None = ..., controls: Any | None = ...
    ): ...
    def abandon(self, message_id, controls: Any | None = ...): ...
    def extended(
        self, request_name, request_value: Any | None = ..., controls: Any | None = ..., no_encode: Any | None = ...
    ): ...
    def start_tls(self, read_server_info: bool = ...): ...
    def do_sasl_bind(self, controls): ...
    def do_ntlm_bind(self, controls): ...
    def refresh_server_info(self) -> None: ...
    def response_to_ldif(
        self,
        search_result: Any | None = ...,
        all_base64: bool = ...,
        line_separator: Any | None = ...,
        sort_order: Any | None = ...,
        stream: Any | None = ...,
    ): ...
    def response_to_json(
        self,
        raw: bool = ...,
        search_result: Any | None = ...,
        indent: int = ...,
        sort: bool = ...,
        stream: Any | None = ...,
        checked_attributes: bool = ...,
        include_empty: bool = ...,
    ): ...
    def response_to_file(self, target, raw: bool = ..., indent: int = ..., sort: bool = ...) -> None: ...
    @property
    def entries(self): ...



# Ideally, we'd just do "from _socket import *". Unfortunately, socket
# overrides some definitions from _socket incompatibly. mypy incorrectly
# prefers the definitions from _socket over those defined here.
import _socket
import sys
from _socket import (
    _FD,
    EAI_AGAIN as EAI_AGAIN,
    EAI_BADFLAGS as EAI_BADFLAGS,
    EAI_FAIL as EAI_FAIL,
    EAI_FAMILY as EAI_FAMILY,
    EAI_MEMORY as EAI_MEMORY,
    EAI_NODATA as EAI_NODATA,
    EAI_NONAME as EAI_NONAME,
    EAI_SERVICE as EAI_SERVICE,
    EAI_SOCKTYPE as EAI_SOCKTYPE,
    INADDR_ALLHOSTS_GROUP as INADDR_ALLHOSTS_GROUP,
    INADDR_ANY as INADDR_ANY,
    INADDR_BROADCAST as INADDR_BROADCAST,
    INADDR_LOOPBACK as INADDR_LOOPBACK,
    INADDR_MAX_LOCAL_GROUP as INADDR_MAX_LOCAL_GROUP,
    INADDR_NONE as INADDR_NONE,
    INADDR_UNSPEC_GROUP as INADDR_UNSPEC_GROUP,
    IP_ADD_MEMBERSHIP as IP_ADD_MEMBERSHIP,
    IP_DROP_MEMBERSHIP as IP_DROP_MEMBERSHIP,
    IP_HDRINCL as IP_HDRINCL,
    IP_MULTICAST_IF as IP_MULTICAST_IF,
    IP_MULTICAST_LOOP as IP_MULTICAST_LOOP,
    IP_MULTICAST_TTL as IP_MULTICAST_TTL,
    IP_OPTIONS as IP_OPTIONS,
    IP_RECVDSTADDR as IP_RECVDSTADDR,
    IP_TOS as IP_TOS,
    IP_TTL as IP_TTL,
    IPPORT_RESERVED as IPPORT_RESERVED,
    IPPORT_USERRESERVED as IPPORT_USERRESERVED,
    IPPROTO_ICMP as IPPROTO_ICMP,
    IPPROTO_IP as IPPROTO_IP,
    IPPROTO_RAW as IPPROTO_RAW,
    IPPROTO_TCP as IPPROTO_TCP,
    IPPROTO_UDP as IPPROTO_UDP,
    IPV6_CHECKSUM as IPV6_CHECKSUM,
    IPV6_JOIN_GROUP as IPV6_JOIN_GROUP,
    IPV6_LEAVE_GROUP as IPV6_LEAVE_GROUP,
    IPV6_MULTICAST_HOPS as IPV6_MULTICAST_HOPS,
    IPV6_MULTICAST_IF as IPV6_MULTICAST_IF,
    IPV6_MULTICAST_LOOP as IPV6_MULTICAST_LOOP,
    IPV6_RECVTCLASS as IPV6_RECVTCLASS,
    IPV6_TCLASS as IPV6_TCLASS,
    IPV6_UNICAST_HOPS as IPV6_UNICAST_HOPS,
    IPV6_V6ONLY as IPV6_V6ONLY,
    NI_DGRAM as NI_DGRAM,
    NI_MAXHOST as NI_MAXHOST,
    NI_MAXSERV as NI_MAXSERV,
    NI_NAMEREQD as NI_NAMEREQD,
    NI_NOFQDN as NI_NOFQDN,
    NI_NUMERICHOST as NI_NUMERICHOST,
    NI_NUMERICSERV as NI_NUMERICSERV,
    SHUT_RD as SHUT_RD,
    SHUT_RDWR as SHUT_RDWR,
    SHUT_WR as SHUT_WR,
    SO_ACCEPTCONN as SO_ACCEPTCONN,
    SO_BROADCAST as SO_BROADCAST,
    SO_DEBUG as SO_DEBUG,
    SO_DONTROUTE as SO_DONTROUTE,
    SO_ERROR as SO_ERROR,
    SO_KEEPALIVE as SO_KEEPALIVE,
    SO_LINGER as SO_LINGER,
    SO_OOBINLINE as SO_OOBINLINE,
    SO_RCVBUF as SO_RCVBUF,
    SO_RCVLOWAT as SO_RCVLOWAT,
    SO_RCVTIMEO as SO_RCVTIMEO,
    SO_REUSEADDR as SO_REUSEADDR,
    SO_SNDBUF as SO_SNDBUF,
    SO_SNDLOWAT as SO_SNDLOWAT,
    SO_SNDTIMEO as SO_SNDTIMEO,
    SO_TYPE as SO_TYPE,
    SO_USELOOPBACK as SO_USELOOPBACK,
    SOL_IP as SOL_IP,
    SOL_SOCKET as SOL_SOCKET,
    SOL_TCP as SOL_TCP,
    SOL_UDP as SOL_UDP,
    SOMAXCONN as SOMAXCONN,
    TCP_FASTOPEN as TCP_FASTOPEN,
    TCP_KEEPCNT as TCP_KEEPCNT,
    TCP_MAXSEG as TCP_MAXSEG,
    TCP_NODELAY as TCP_NODELAY,
    SocketType as SocketType,
    _Address as _Address,
    _RetAddress as _RetAddress,
    dup as dup,
    error as error,
    gaierror as gaierror,
    getdefaulttimeout as getdefaulttimeout,
    gethostbyaddr as gethostbyaddr,
    gethostbyname as gethostbyname,
    gethostbyname_ex as gethostbyname_ex,
    gethostname as gethostname,
    getnameinfo as getnameinfo,
    getprotobyname as getprotobyname,
    getservbyname as getservbyname,
    getservbyport as getservbyport,
    has_ipv6 as has_ipv6,
    herror as herror,
    htonl as htonl,
    htons as htons,
    inet_aton as inet_aton,
    inet_ntoa as inet_ntoa,
    inet_ntop as inet_ntop,
    inet_pton as inet_pton,
    ntohl as ntohl,
    ntohs as ntohs,
    setdefaulttimeout as setdefaulttimeout,
    timeout as timeout,
)
from _typeshed import ReadableBuffer, Self, WriteableBuffer
from collections.abc import Iterable
from enum import IntEnum, IntFlag
from io import BufferedReader, BufferedRWPair, BufferedWriter, IOBase, RawIOBase, TextIOWrapper
from typing import Any, Protocol, overload
from typing_extensions import Literal

if sys.platform != "darwin" or sys.version_info >= (3, 9):
    from _socket import (
        IPV6_DONTFRAG as IPV6_DONTFRAG,
        IPV6_HOPLIMIT as IPV6_HOPLIMIT,
        IPV6_HOPOPTS as IPV6_HOPOPTS,
        IPV6_PKTINFO as IPV6_PKTINFO,
        IPV6_RECVRTHDR as IPV6_RECVRTHDR,
        IPV6_RTHDR as IPV6_RTHDR,
    )

if sys.platform != "darwin":
    from _socket import SO_EXCLUSIVEADDRUSE as SO_EXCLUSIVEADDRUSE

if sys.version_info >= (3, 10):
    from _socket import IP_RECVTOS as IP_RECVTOS
elif sys.platform != "darwin" and sys.platform != "win32":
    from _socket import IP_RECVTOS as IP_RECVTOS

from _socket import TCP_KEEPINTVL as TCP_KEEPINTVL, close as close

if sys.platform != "darwin":
    from _socket import TCP_KEEPIDLE as TCP_KEEPIDLE

if sys.platform != "win32" or sys.version_info >= (3, 8):
    from _socket import (
        IPPROTO_AH as IPPROTO_AH,
        IPPROTO_DSTOPTS as IPPROTO_DSTOPTS,
        IPPROTO_EGP as IPPROTO_EGP,
        IPPROTO_ESP as IPPROTO_ESP,
        IPPROTO_FRAGMENT as IPPROTO_FRAGMENT,
        IPPROTO_GGP as IPPROTO_GGP,
        IPPROTO_HOPOPTS as IPPROTO_HOPOPTS,
        IPPROTO_ICMPV6 as IPPROTO_ICMPV6,
        IPPROTO_IDP as IPPROTO_IDP,
        IPPROTO_IGMP as IPPROTO_IGMP,
        IPPROTO_IPV4 as IPPROTO_IPV4,
        IPPROTO_IPV6 as IPPROTO_IPV6,
        IPPROTO_MAX as IPPROTO_MAX,
        IPPROTO_ND as IPPROTO_ND,
        IPPROTO_NONE as IPPROTO_NONE,
        IPPROTO_PIM as IPPROTO_PIM,
        IPPROTO_PUP as IPPROTO_PUP,
        IPPROTO_ROUTING as IPPROTO_ROUTING,
        IPPROTO_SCTP as IPPROTO_SCTP,
    )

    if sys.platform != "darwin":
        from _socket import (
            IPPROTO_CBT as IPPROTO_CBT,
            IPPROTO_ICLFXBM as IPPROTO_ICLFXBM,
            IPPROTO_IGP as IPPROTO_IGP,
            IPPROTO_L2TP as IPPROTO_L2TP,
            IPPROTO_PGM as IPPROTO_PGM,
            IPPROTO_RDP as IPPROTO_RDP,
            IPPROTO_ST as IPPROTO_ST,
        )
if sys.platform != "win32" and sys.platform != "darwin":
    from _socket import (
        IP_TRANSPARENT as IP_TRANSPARENT,
        IPPROTO_BIP as IPPROTO_BIP,
        IPPROTO_MOBILE as IPPROTO_MOBILE,
        IPPROTO_VRRP as IPPROTO_VRRP,
        IPX_TYPE as IPX_TYPE,
        SCM_CREDENTIALS as SCM_CREDENTIALS,
        SO_BINDTODEVICE as SO_BINDTODEVICE,
        SO_MARK as SO_MARK,
        SO_PASSCRED as SO_PASSCRED,
        SO_PEERCRED as SO_PEERCRED,
        SO_PRIORITY as SO_PRIORITY,
        SO_SETFIB as SO_SETFIB,
        SOL_ATALK as SOL_ATALK,
        SOL_AX25 as SOL_AX25,
        SOL_HCI as SOL_HCI,
        SOL_IPX as SOL_IPX,
        SOL_NETROM as SOL_NETROM,
        SOL_ROSE as SOL_ROSE,
        TCP_CORK as TCP_CORK,
        TCP_DEFER_ACCEPT as TCP_DEFER_ACCEPT,
        TCP_INFO as TCP_INFO,
        TCP_LINGER2 as TCP_LINGER2,
        TCP_QUICKACK as TCP_QUICKACK,
        TCP_SYNCNT as TCP_SYNCNT,
        TCP_WINDOW_CLAMP as TCP_WINDOW_CLAMP,
    )
if sys.platform != "win32":
    from _socket import (
        CMSG_LEN as CMSG_LEN,
        CMSG_SPACE as CMSG_SPACE,
        EAI_ADDRFAMILY as EAI_ADDRFAMILY,
        EAI_BADHINTS as EAI_BADHINTS,
        EAI_MAX as EAI_MAX,
        EAI_OVERFLOW as EAI_OVERFLOW,
        EAI_PROTOCOL as EAI_PROTOCOL,
        EAI_SYSTEM as EAI_SYSTEM,
        IP_DEFAULT_MULTICAST_LOOP as IP_DEFAULT_MULTICAST_LOOP,
        IP_DEFAULT_MULTICAST_TTL as IP_DEFAULT_MULTICAST_TTL,
        IP_MAX_MEMBERSHIPS as IP_MAX_MEMBERSHIPS,
        IP_RECVOPTS as IP_RECVOPTS,
        IP_RECVRETOPTS as IP_RECVRETOPTS,
        IP_RETOPTS as IP_RETOPTS,
        IPPROTO_EON as IPPROTO_EON,
        IPPROTO_GRE as IPPROTO_GRE,
        IPPROTO_HELLO as IPPROTO_HELLO,
        IPPROTO_IPCOMP as IPPROTO_IPCOMP,
        IPPROTO_IPIP as IPPROTO_IPIP,
        IPPROTO_RSVP as IPPROTO_RSVP,
        IPPROTO_TP as IPPROTO_TP,
        IPPROTO_XTP as IPPROTO_XTP,
        IPV6_RTHDR_TYPE_0 as IPV6_RTHDR_TYPE_0,
        LOCAL_PEERCRED as LOCAL_PEERCRED,
        SCM_CREDS as SCM_CREDS,
        SCM_RIGHTS as SCM_RIGHTS,
        SO_REUSEPORT as SO_REUSEPORT,
        sethostname as sethostname,
    )

    if sys.platform != "darwin" or sys.version_info >= (3, 9):
        from _socket import (
            IPV6_DSTOPTS as IPV6_DSTOPTS,
            IPV6_NEXTHOP as IPV6_NEXTHOP,
            IPV6_PATHMTU as IPV6_PATHMTU,
            IPV6_RECVDSTOPTS as IPV6_RECVDSTOPTS,
            IPV6_RECVHOPLIMIT as IPV6_RECVHOPLIMIT,
            IPV6_RECVHOPOPTS as IPV6_RECVHOPOPTS,
            IPV6_RECVPATHMTU as IPV6_RECVPATHMTU,
            IPV6_RECVPKTINFO as IPV6_RECVPKTINFO,
            IPV6_RTHDRDSTOPTS as IPV6_RTHDRDSTOPTS,
            IPV6_USE_MIN_MTU as IPV6_USE_MIN_MTU,
        )

if sys.platform != "win32" or sys.version_info >= (3, 8):
    from _socket import if_indextoname as if_indextoname, if_nameindex as if_nameindex, if_nametoindex as if_nametoindex

if sys.platform != "darwin":
    if sys.platform != "win32" or sys.version_info >= (3, 9):
        from _socket import BDADDR_ANY as BDADDR_ANY, BDADDR_LOCAL as BDADDR_LOCAL, BTPROTO_RFCOMM as BTPROTO_RFCOMM

if sys.platform == "darwin" and sys.version_info >= (3, 10):
    from _socket import TCP_KEEPALIVE as TCP_KEEPALIVE

if sys.platform == "linux":
    from _socket import (
        ALG_OP_DECRYPT as ALG_OP_DECRYPT,
        ALG_OP_ENCRYPT as ALG_OP_ENCRYPT,
        ALG_OP_SIGN as ALG_OP_SIGN,
        ALG_OP_VERIFY as ALG_OP_VERIFY,
        ALG_SET_AEAD_ASSOCLEN as ALG_SET_AEAD_ASSOCLEN,
        ALG_SET_AEAD_AUTHSIZE as ALG_SET_AEAD_AUTHSIZE,
        ALG_SET_IV as ALG_SET_IV,
        ALG_SET_KEY as ALG_SET_KEY,
        ALG_SET_OP as ALG_SET_OP,
        ALG_SET_PUBKEY as ALG_SET_PUBKEY,
        CAN_BCM as CAN_BCM,
        CAN_BCM_RX_CHANGED as CAN_BCM_RX_CHANGED,
        CAN_BCM_RX_DELETE as CAN_BCM_RX_DELETE,
        CAN_BCM_RX_READ as CAN_BCM_RX_READ,
        CAN_BCM_RX_SETUP as CAN_BCM_RX_SETUP,
        CAN_BCM_RX_STATUS as CAN_BCM_RX_STATUS,
        CAN_BCM_RX_TIMEOUT as CAN_BCM_RX_TIMEOUT,
        CAN_BCM_TX_DELETE as CAN_BCM_TX_DELETE,
        CAN_BCM_TX_EXPIRED as CAN_BCM_TX_EXPIRED,
        CAN_BCM_TX_READ as CAN_BCM_TX_READ,
        CAN_BCM_TX_SEND as CAN_BCM_TX_SEND,
        CAN_BCM_TX_SETUP as CAN_BCM_TX_SETUP,
        CAN_BCM_TX_STATUS as CAN_BCM_TX_STATUS,
        CAN_EFF_FLAG as CAN_EFF_FLAG,
        CAN_EFF_MASK as CAN_EFF_MASK,
        CAN_ERR_FLAG as CAN_ERR_FLAG,
        CAN_ERR_MASK as CAN_ERR_MASK,
        CAN_RAW as CAN_RAW,
        CAN_RAW_ERR_FILTER as CAN_RAW_ERR_FILTER,
        CAN_RAW_FD_FRAMES as CAN_RAW_FD_FRAMES,
        CAN_RAW_FILTER as CAN_RAW_FILTER,
        CAN_RAW_LOOPBACK as CAN_RAW_LOOPBACK,
        CAN_RAW_RECV_OWN_MSGS as CAN_RAW_RECV_OWN_MSGS,
        CAN_RTR_FLAG as CAN_RTR_FLAG,
        CAN_SFF_MASK as CAN_SFF_MASK,
        NETLINK_ARPD as NETLINK_ARPD,
        NETLINK_CRYPTO as NETLINK_CRYPTO,
        NETLINK_DNRTMSG as NETLINK_DNRTMSG,
        NETLINK_FIREWALL as NETLINK_FIREWALL,
        NETLINK_IP6_FW as NETLINK_IP6_FW,
        NETLINK_NFLOG as NETLINK_NFLOG,
        NETLINK_ROUTE as NETLINK_ROUTE,
        NETLINK_ROUTE6 as NETLINK_ROUTE6,
        NETLINK_SKIP as NETLINK_SKIP,
        NETLINK_TAPBASE as NETLINK_TAPBASE,
        NETLINK_TCPDIAG as NETLINK_TCPDIAG,
        NETLINK_USERSOCK as NETLINK_USERSOCK,
        NETLINK_W1 as NETLINK_W1,
        NETLINK_XFRM as NETLINK_XFRM,
        PACKET_BROADCAST as PACKET_BROADCAST,
        PACKET_FASTROUTE as PACKET_FASTROUTE,
        PACKET_HOST as PACKET_HOST,
        PACKET_LOOPBACK as PACKET_LOOPBACK,
        PACKET_MULTICAST as PACKET_MULTICAST,
        PACKET_OTHERHOST as PACKET_OTHERHOST,
        PACKET_OUTGOING as PACKET_OUTGOING,
        PF_CAN as PF_CAN,
        PF_PACKET as PF_PACKET,
        PF_RDS as PF_RDS,
        RDS_CANCEL_SENT_TO as RDS_CANCEL_SENT_TO,
        RDS_CMSG_RDMA_ARGS as RDS_CMSG_RDMA_ARGS,
        RDS_CMSG_RDMA_DEST as RDS_CMSG_RDMA_DEST,
        RDS_CMSG_RDMA_MAP as RDS_CMSG_RDMA_MAP,
        RDS_CMSG_RDMA_STATUS as RDS_CMSG_RDMA_STATUS,
        RDS_CMSG_RDMA_UPDATE as RDS_CMSG_RDMA_UPDATE,
        RDS_CONG_MONITOR as RDS_CONG_MONITOR,
        RDS_FREE_MR as RDS_FREE_MR,
        RDS_GET_MR as RDS_GET_MR,
        RDS_GET_MR_FOR_DEST as RDS_GET_MR_FOR_DEST,
        RDS_RDMA_DONTWAIT as RDS_RDMA_DONTWAIT,
        RDS_RDMA_FENCE as RDS_RDMA_FENCE,
        RDS_RDMA_INVALIDATE as RDS_RDMA_INVALIDATE,
        RDS_RDMA_NOTIFY_ME as RDS_RDMA_NOTIFY_ME,
        RDS_RDMA_READWRITE as RDS_RDMA_READWRITE,
        RDS_RDMA_SILENT as RDS_RDMA_SILENT,
        RDS_RDMA_USE_ONCE as RDS_RDMA_USE_ONCE,
        RDS_RECVERR as RDS_RECVERR,
        SOL_ALG as SOL_ALG,
        SOL_CAN_BASE as SOL_CAN_BASE,
        SOL_CAN_RAW as SOL_CAN_RAW,
        SOL_RDS as SOL_RDS,
        SOL_TIPC as SOL_TIPC,
        TIPC_ADDR_ID as TIPC_ADDR_ID,
        TIPC_ADDR_NAME as TIPC_ADDR_NAME,
        TIPC_ADDR_NAMESEQ as TIPC_ADDR_NAMESEQ,
        TIPC_CFG_SRV as TIPC_CFG_SRV,
        TIPC_CLUSTER_SCOPE as TIPC_CLUSTER_SCOPE,
        TIPC_CONN_TIMEOUT as TIPC_CONN_TIMEOUT,
        TIPC_CRITICAL_IMPORTANCE as TIPC_CRITICAL_IMPORTANCE,
        TIPC_DEST_DROPPABLE as TIPC_DEST_DROPPABLE,
        TIPC_HIGH_IMPORTANCE as TIPC_HIGH_IMPORTANCE,
        TIPC_IMPORTANCE as TIPC_IMPORTANCE,
        TIPC_LOW_IMPORTANCE as TIPC_LOW_IMPORTANCE,
        TIPC_MEDIUM_IMPORTANCE as TIPC_MEDIUM_IMPORTANCE,
        TIPC_NODE_SCOPE as TIPC_NODE_SCOPE,
        TIPC_PUBLISHED as TIPC_PUBLISHED,
        TIPC_SRC_DROPPABLE as TIPC_SRC_DROPPABLE,
        TIPC_SUB_CANCEL as TIPC_SUB_CANCEL,
        TIPC_SUB_PORTS as TIPC_SUB_PORTS,
        TIPC_SUB_SERVICE as TIPC_SUB_SERVICE,
        TIPC_SUBSCR_TIMEOUT as TIPC_SUBSCR_TIMEOUT,
        TIPC_TOP_SRV as TIPC_TOP_SRV,
        TIPC_WAIT_FOREVER as TIPC_WAIT_FOREVER,
        TIPC_WITHDRAWN as TIPC_WITHDRAWN,
        TIPC_ZONE_SCOPE as TIPC_ZONE_SCOPE,
    )
if sys.platform == "linux":
    from _socket import (
        CAN_ISOTP as CAN_ISOTP,
        IOCTL_VM_SOCKETS_GET_LOCAL_CID as IOCTL_VM_SOCKETS_GET_LOCAL_CID,
        SO_VM_SOCKETS_BUFFER_MAX_SIZE as SO_VM_SOCKETS_BUFFER_MAX_SIZE,
        SO_VM_SOCKETS_BUFFER_MIN_SIZE as SO_VM_SOCKETS_BUFFER_MIN_SIZE,
        SO_VM_SOCKETS_BUFFER_SIZE as SO_VM_SOCKETS_BUFFER_SIZE,
        VM_SOCKETS_INVALID_VERSION as VM_SOCKETS_INVALID_VERSION,
        VMADDR_CID_ANY as VMADDR_CID_ANY,
        VMADDR_CID_HOST as VMADDR_CID_HOST,
        VMADDR_PORT_ANY as VMADDR_PORT_ANY,
    )
if sys.platform != "win32":
    from _socket import TCP_NOTSENT_LOWAT as TCP_NOTSENT_LOWAT
if sys.platform == "linux" and sys.version_info >= (3, 8):
    from _socket import (
        CAN_BCM_CAN_FD_FRAME as CAN_BCM_CAN_FD_FRAME,
        CAN_BCM_RX_ANNOUNCE_RESUME as CAN_BCM_RX_ANNOUNCE_RESUME,
        CAN_BCM_RX_CHECK_DLC as CAN_BCM_RX_CHECK_DLC,
        CAN_BCM_RX_FILTER_ID as CAN_BCM_RX_FILTER_ID,
        CAN_BCM_RX_NO_AUTOTIMER as CAN_BCM_RX_NO_AUTOTIMER,
        CAN_BCM_RX_RTR_FRAME as CAN_BCM_RX_RTR_FRAME,
        CAN_BCM_SETTIMER as CAN_BCM_SETTIMER,
        CAN_BCM_STARTTIMER as CAN_BCM_STARTTIMER,
        CAN_BCM_TX_ANNOUNCE as CAN_BCM_TX_ANNOUNCE,
        CAN_BCM_TX_COUNTEVT as CAN_BCM_TX_COUNTEVT,
        CAN_BCM_TX_CP_CAN_ID as CAN_BCM_TX_CP_CAN_ID,
        CAN_BCM_TX_RESET_MULTI_IDX as CAN_BCM_TX_RESET_MULTI_IDX,
    )
if sys.platform == "linux" and sys.version_info >= (3, 9):
    from _socket import (
        CAN_J1939 as CAN_J1939,
        CAN_RAW_JOIN_FILTERS as CAN_RAW_JOIN_FILTERS,
        J1939_EE_INFO_NONE as J1939_EE_INFO_NONE,
        J1939_EE_INFO_TX_ABORT as J1939_EE_INFO_TX_ABORT,
        J1939_FILTER_MAX as J1939_FILTER_MAX,
        J1939_IDLE_ADDR as J1939_IDLE_ADDR,
        J1939_MAX_UNICAST_ADDR as J1939_MAX_UNICAST_ADDR,
        J1939_NLA_BYTES_ACKED as J1939_NLA_BYTES_ACKED,
        J1939_NLA_PAD as J1939_NLA_PAD,
        J1939_NO_ADDR as J1939_NO_ADDR,
        J1939_NO_NAME as J1939_NO_NAME,
        J1939_NO_PGN as J1939_NO_PGN,
        J1939_PGN_ADDRESS_CLAIMED as J1939_PGN_ADDRESS_CLAIMED,
        J1939_PGN_ADDRESS_COMMANDED as J1939_PGN_ADDRESS_COMMANDED,
        J1939_PGN_MAX as J1939_PGN_MAX,
        J1939_PGN_PDU1_MAX as J1939_PGN_PDU1_MAX,
        J1939_PGN_REQUEST as J1939_PGN_REQUEST,
        SCM_J1939_DEST_ADDR as SCM_J1939_DEST_ADDR,
        SCM_J1939_DEST_NAME as SCM_J1939_DEST_NAME,
        SCM_J1939_ERRQUEUE as SCM_J1939_ERRQUEUE,
        SCM_J1939_PRIO as SCM_J1939_PRIO,
        SO_J1939_ERRQUEUE as SO_J1939_ERRQUEUE,
        SO_J1939_FILTER as SO_J1939_FILTER,
        SO_J1939_PROMISC as SO_J1939_PROMISC,
        SO_J1939_SEND_PRIO as SO_J1939_SEND_PRIO,
    )
if sys.platform == "linux" and sys.version_info >= (3, 10):
    from _socket import IPPROTO_MPTCP as IPPROTO_MPTCP
if sys.platform == "linux" and sys.version_info >= (3, 11):
    from _socket import SO_INCOMING_CPU as SO_INCOMING_CPU
if sys.platform == "win32":
    from _socket import (
        RCVALL_MAX as RCVALL_MAX,
        RCVALL_OFF as RCVALL_OFF,
        RCVALL_ON as RCVALL_ON,
        RCVALL_SOCKETLEVELONLY as RCVALL_SOCKETLEVELONLY,
        SIO_KEEPALIVE_VALS as SIO_KEEPALIVE_VALS,
        SIO_LOOPBACK_FAST_PATH as SIO_LOOPBACK_FAST_PATH,
        SIO_RCVALL as SIO_RCVALL,
    )

# Re-exported from errno
EBADF: int
EAGAIN: int
EWOULDBLOCK: int

class AddressFamily(IntEnum):
    AF_INET: int
    AF_INET6: int
    AF_APPLETALK: int
    AF_DECnet: int
    AF_IPX: int
    AF_SNA: int
    AF_UNSPEC: int
    if sys.platform != "darwin":
        AF_IRDA: int
    if sys.platform != "win32":
        AF_ROUTE: int
        AF_SYSTEM: int
        AF_UNIX: int
    if sys.platform != "darwin" and sys.platform != "win32":
        AF_AAL5: int
        AF_ASH: int
        AF_ATMPVC: int
        AF_ATMSVC: int
        AF_AX25: int
        AF_BRIDGE: int
        AF_ECONET: int
        AF_KEY: int
        AF_LLC: int
        AF_NETBEUI: int
        AF_NETROM: int
        AF_PPPOX: int
        AF_ROSE: int
        AF_SECURITY: int
        AF_WANPIPE: int
        AF_X25: int
    if sys.platform == "linux":
        AF_CAN: int
        AF_PACKET: int
        AF_RDS: int
        AF_TIPC: int
        AF_ALG: int
        AF_NETLINK: int
        AF_VSOCK: int
        if sys.version_info >= (3, 8):
            AF_QIPCRTR: int
    if sys.platform != "win32" or sys.version_info >= (3, 9):
        AF_LINK: int
        if sys.platform != "darwin":
            AF_BLUETOOTH: int

AF_INET = AddressFamily.AF_INET
AF_INET6 = AddressFamily.AF_INET6
AF_APPLETALK = AddressFamily.AF_APPLETALK
AF_DECnet = AddressFamily.AF_DECnet
AF_IPX = AddressFamily.AF_IPX
AF_SNA = AddressFamily.AF_SNA
AF_UNSPEC = AddressFamily.AF_UNSPEC

if sys.platform != "darwin":
    AF_IRDA = AddressFamily.AF_IRDA

if sys.platform != "win32":
    AF_ROUTE = AddressFamily.AF_ROUTE
    AF_SYSTEM = AddressFamily.AF_SYSTEM
    AF_UNIX = AddressFamily.AF_UNIX

if sys.platform != "win32" and sys.platform != "darwin":
    AF_AAL5 = AddressFamily.AF_AAL5
    AF_ASH = AddressFamily.AF_ASH
    AF_ATMPVC = AddressFamily.AF_ATMPVC
    AF_ATMSVC = AddressFamily.AF_ATMSVC
    AF_AX25 = AddressFamily.AF_AX25
    AF_BRIDGE = AddressFamily.AF_BRIDGE
    AF_ECONET = AddressFamily.AF_ECONET
    AF_KEY = AddressFamily.AF_KEY
    AF_LLC = AddressFamily.AF_LLC
    AF_NETBEUI = AddressFamily.AF_NETBEUI
    AF_NETROM = AddressFamily.AF_NETROM
    AF_PPPOX = AddressFamily.AF_PPPOX
    AF_ROSE = AddressFamily.AF_ROSE
    AF_SECURITY = AddressFamily.AF_SECURITY
    AF_WANPIPE = AddressFamily.AF_WANPIPE
    AF_X25 = AddressFamily.AF_X25

if sys.platform == "linux":
    AF_CAN = AddressFamily.AF_CAN
    AF_PACKET = AddressFamily.AF_PACKET
    AF_RDS = AddressFamily.AF_RDS
    AF_TIPC = AddressFamily.AF_TIPC
    AF_ALG = AddressFamily.AF_ALG
    AF_NETLINK = AddressFamily.AF_NETLINK
    AF_VSOCK = AddressFamily.AF_VSOCK
    if sys.version_info >= (3, 8):
        AF_QIPCRTR = AddressFamily.AF_QIPCRTR

if sys.platform != "win32" or sys.version_info >= (3, 9):
    AF_LINK = AddressFamily.AF_LINK
    if sys.platform != "darwin":
        AF_BLUETOOTH = AddressFamily.AF_BLUETOOTH

class SocketKind(IntEnum):
    SOCK_STREAM: int
    SOCK_DGRAM: int
    SOCK_RAW: int
    SOCK_RDM: int
    SOCK_SEQPACKET: int
    if sys.platform == "linux":
        SOCK_CLOEXEC: int
        SOCK_NONBLOCK: int

SOCK_STREAM = SocketKind.SOCK_STREAM
SOCK_DGRAM = SocketKind.SOCK_DGRAM
SOCK_RAW = SocketKind.SOCK_RAW
SOCK_RDM = SocketKind.SOCK_RDM
SOCK_SEQPACKET = SocketKind.SOCK_SEQPACKET
if sys.platform == "linux":
    SOCK_CLOEXEC = SocketKind.SOCK_CLOEXEC
    SOCK_NONBLOCK = SocketKind.SOCK_NONBLOCK

class MsgFlag(IntFlag):
    MSG_CTRUNC: int
    MSG_DONTROUTE: int
    MSG_OOB: int
    MSG_PEEK: int
    MSG_TRUNC: int
    MSG_WAITALL: int

    if sys.platform != "darwin":
        MSG_BCAST: int
        MSG_MCAST: int
        MSG_ERRQUEUE: int

    if sys.platform != "win32" and sys.platform != "darwin":
        MSG_BTAG: int
        MSG_CMSG_CLOEXEC: int
        MSG_CONFIRM: int
        MSG_ETAG: int
        MSG_FASTOPEN: int
        MSG_MORE: int
        MSG_NOTIFICATION: int

    if sys.platform != "win32":
        MSG_DONTWAIT: int
        MSG_EOF: int
        MSG_EOR: int
        MSG_NOSIGNAL: int  # sometimes this exists on darwin, sometimes not

MSG_CTRUNC = MsgFlag.MSG_CTRUNC
MSG_DONTROUTE = MsgFlag.MSG_DONTROUTE
MSG_OOB = MsgFlag.MSG_OOB
MSG_PEEK = MsgFlag.MSG_PEEK
MSG_TRUNC = MsgFlag.MSG_TRUNC
MSG_WAITALL = MsgFlag.MSG_WAITALL

if sys.platform != "darwin":
    MSG_BCAST = MsgFlag.MSG_BCAST
    MSG_MCAST = MsgFlag.MSG_MCAST
    MSG_ERRQUEUE = MsgFlag.MSG_ERRQUEUE

if sys.platform != "win32":
    MSG_DONTWAIT = MsgFlag.MSG_DONTWAIT
    MSG_EOF = MsgFlag.MSG_EOF
    MSG_EOR = MsgFlag.MSG_EOR
    MSG_NOSIGNAL = MsgFlag.MSG_NOSIGNAL  # Sometimes this exists on darwin, sometimes not

if sys.platform != "win32" and sys.platform != "darwin":
    MSG_BTAG = MsgFlag.MSG_BTAG
    MSG_CMSG_CLOEXEC = MsgFlag.MSG_CMSG_CLOEXEC
    MSG_CONFIRM = MsgFlag.MSG_CONFIRM
    MSG_ETAG = MsgFlag.MSG_ETAG
    MSG_FASTOPEN = MsgFlag.MSG_FASTOPEN
    MSG_MORE = MsgFlag.MSG_MORE
    MSG_NOTIFICATION = MsgFlag.MSG_NOTIFICATION

class AddressInfo(IntFlag):
    AI_ADDRCONFIG: int
    AI_ALL: int
    AI_CANONNAME: int
    AI_NUMERICHOST: int
    AI_NUMERICSERV: int
    AI_PASSIVE: int
    AI_V4MAPPED: int
    if sys.platform != "win32":
        AI_DEFAULT: int
        AI_MASK: int
        AI_V4MAPPED_CFG: int

AI_ADDRCONFIG = AddressInfo.AI_ADDRCONFIG
AI_ALL = AddressInfo.AI_ALL
AI_CANONNAME = AddressInfo.AI_CANONNAME
AI_NUMERICHOST = AddressInfo.AI_NUMERICHOST
AI_NUMERICSERV = AddressInfo.AI_NUMERICSERV
AI_PASSIVE = AddressInfo.AI_PASSIVE
AI_V4MAPPED = AddressInfo.AI_V4MAPPED

if sys.platform != "win32":
    AI_DEFAULT = AddressInfo.AI_DEFAULT
    AI_MASK = AddressInfo.AI_MASK
    AI_V4MAPPED_CFG = AddressInfo.AI_V4MAPPED_CFG

if sys.platform == "win32":
    errorTab: dict[int, str]  # undocumented

class _SendableFile(Protocol):
    def read(self, __size: int) -> bytes: ...
    def seek(self, __offset: int) -> object: ...

    # optional fields:
    #
    # @property
    # def mode(self) -> str: ...
    # def fileno(self) -> int: ...

class socket(_socket.socket):
    def __init__(
        self, family: AddressFamily | int = ..., type: SocketKind | int = ..., proto: int = ..., fileno: int | None = ...
    ) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def dup(self: Self) -> Self: ...  # noqa: F811
    def accept(self) -> tuple[socket, _RetAddress]: ...
    # Note that the makefile's documented windows-specific behavior is not represented
    # mode strings with duplicates are intentionally excluded
    @overload
    def makefile(  # type: ignore[misc]
        self,
        mode: Literal["b", "rb", "br", "wb", "bw", "rwb", "rbw", "wrb", "wbr", "brw", "bwr"],
        buffering: Literal[0],
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> SocketIO: ...
    @overload
    def makefile(
        self,
        mode: Literal["rwb", "rbw", "wrb", "wbr", "brw", "bwr"],
        buffering: Literal[-1, 1] | None = ...,
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> BufferedRWPair: ...
    @overload
    def makefile(
        self,
        mode: Literal["rb", "br"],
        buffering: Literal[-1, 1] | None = ...,
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> BufferedReader: ...
    @overload
    def makefile(
        self,
        mode: Literal["wb", "bw"],
        buffering: Literal[-1, 1] | None = ...,
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> BufferedWriter: ...
    @overload
    def makefile(
        self,
        mode: Literal["b", "rb", "br", "wb", "bw", "rwb", "rbw", "wrb", "wbr", "brw", "bwr"],
        buffering: int,
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> IOBase: ...
    @overload
    def makefile(
        self,
        mode: Literal["r", "w", "rw", "wr", ""] = ...,
        buffering: int | None = ...,
        *,
        encoding: str | None = ...,
        errors: str | None = ...,
        newline: str | None = ...,
    ) -> TextIOWrapper: ...
    def sendfile(self, file: _SendableFile, offset: int = ..., count: int | None = ...) -> int: ...
    @property
    def family(self) -> AddressFamily: ...  # type: ignore[override]
    @property
    def type(self) -> SocketKind: ...  # type: ignore[override]
    def get_inheritable(self) -> bool: ...
    def set_inheritable(self, inheritable: bool) -> None: ...

def fromfd(fd: _FD, family: AddressFamily | int, type: SocketKind | int, proto: int = ...) -> socket: ...

if sys.platform != "win32":
    if sys.version_info >= (3, 9):
        # flags and address appear to be unused in send_fds and recv_fds
        def send_fds(
            sock: socket, buffers: Iterable[ReadableBuffer], fds: Iterable[int], flags: int = ..., address: None = ...
        ) -> int: ...
        def recv_fds(sock: socket, bufsize: int, maxfds: int, flags: int = ...) -> tuple[bytes, list[int], int, Any]: ...

if sys.platform == "win32":
    def fromshare(info: bytes) -> socket: ...

if sys.platform == "win32":
    def socketpair(family: int = ..., type: int = ..., proto: int = ...) -> tuple[socket, socket]: ...

else:
    def socketpair(
        family: int | AddressFamily | None = ..., type: SocketType | int = ..., proto: int = ...
    ) -> tuple[socket, socket]: ...

class SocketIO(RawIOBase):
    def __init__(self, sock: socket, mode: Literal["r", "w", "rw", "rb", "wb", "rwb"]) -> None: ...
    def readinto(self, b: WriteableBuffer) -> int | None: ...
    def write(self, b: ReadableBuffer) -> int | None: ...
    @property
    def name(self) -> int: ...  # return value is really "int"
    @property
    def mode(self) -> Literal["rb", "wb", "rwb"]: ...

def getfqdn(name: str = ...) -> str: ...

if sys.version_info >= (3, 11):
    def create_connection(
        address: tuple[str | None, int],
        timeout: float | None = ...,  # noqa: F811
        source_address: _Address | None = ...,
        *,
        all_errors: bool = ...,
    ) -> socket: ...

else:
    def create_connection(
        address: tuple[str | None, int], timeout: float | None = ..., source_address: _Address | None = ...  # noqa: F811
    ) -> socket: ...

if sys.version_info >= (3, 8):
    def has_dualstack_ipv6() -> bool: ...
    def create_server(
        address: _Address, *, family: int = ..., backlog: int | None = ..., reuse_port: bool = ..., dualstack_ipv6: bool = ...
    ) -> socket: ...

# the 5th tuple item is an address
def getaddrinfo(
    host: bytes | str | None,
    port: bytes | str | int | None,
    family: int = ...,
    type: int = ...,
    proto: int = ...,
    flags: int = ...,
) -> list[tuple[AddressFamily, SocketKind, int, str, tuple[str, int] | tuple[str, int, int, int]]]: ...



from html.parser import HTMLParser
from typing import Any

__author__: str
__copyright__: str
__license__: str

LOGGER: Any
BULLET_WIN1252: str
DEFAULT_HEADING_SIZES: Any
COLOR_DICT: Any

def px2mm(px): ...
def color_as_decimal(color: str = ...): ...

class HTML2FPDF(HTMLParser):
    pdf: Any
    image_map: Any
    li_tag_indent: Any
    table_line_separators: Any
    ul_bullet_char: Any
    style: Any
    href: str
    align: str
    page_links: Any
    font_stack: Any
    indent: int
    bullet: Any
    font_size: Any
    font_color: Any
    table: Any
    table_col_width: Any
    table_col_index: Any
    td: Any
    th: Any
    tr: Any
    thead: Any
    tfoot: Any
    tr_index: Any
    theader: Any
    tfooter: Any
    theader_out: bool
    table_row_height: int
    heading_level: Any
    heading_sizes: Any
    heading_above: float
    heading_below: float
    def __init__(
        self,
        pdf,
        image_map: Any | None = ...,
        li_tag_indent: int = ...,
        table_line_separators: bool = ...,
        ul_bullet_char=...,
        heading_sizes: Any | None = ...,
        **_,
    ): ...
    def width2unit(self, length): ...
    def handle_data(self, data) -> None: ...
    def box_shadow(self, w, h, bgcolor) -> None: ...
    def output_table_header(self) -> None: ...
    tfooter_out: bool
    def output_table_footer(self) -> None: ...
    def output_table_sep(self) -> None: ...
    font_face: Any
    table_offset: Any
    def handle_starttag(self, tag, attrs) -> None: ...
    tbody: Any
    def handle_endtag(self, tag) -> None: ...
    h: Any
    def set_font(self, face: Any | None = ..., size: Any | None = ...) -> None: ...
    def set_style(self, tag: Any | None = ..., enable: bool = ...) -> None: ...
    def set_text_color(self, r: Any | None = ..., g: int = ..., b: int = ...) -> None: ...
    def put_link(self, txt) -> None: ...
    def render_toc(self, pdf, outline) -> None: ...
    def error(self, message: str) -> None: ...

class HTMLMixin:
    HTML2FPDF_CLASS: Any
    def write_html(self, text, *args, **kwargs) -> None: ...



from _typeshed import Incomplete
from abc import ABC

from .syntax import PDFObject

class Action(ABC):
    next: PDFObject | str | None
    def __init__(self, next_action: PDFObject | str | None = ...) -> None: ...
    def dict_as_string(self, key_values: dict[str, Incomplete] | None = ...) -> str: ...

class NamedAction(Action):
    action_name: Incomplete
    def __init__(self, action_name, next_action: PDFObject | str | None = ...) -> None: ...
    def dict_as_string(self) -> str: ...  # type: ignore[override]

class GoToAction(Action):
    dest: Incomplete
    def __init__(self, dest, next_action: PDFObject | str | None = ...) -> None: ...
    def dict_as_string(self) -> str: ...  # type: ignore[override]

class GoToRemoteAction(Action):
    file: Incomplete
    dest: Incomplete
    def __init__(self, file, dest, next_action: PDFObject | str | None = ...) -> None: ...
    def dict_as_string(self) -> str: ...  # type: ignore[override]

class LaunchAction(Action):
    file: Incomplete
    def __init__(self, file, next_action: PDFObject | str | None = ...) -> None: ...
    def dict_as_string(self) -> str: ...  # type: ignore[override]



from . import _LangModelType

Latin5_BulgarianCharToOrderMap: tuple[int, ...]
win1251BulgarianCharToOrderMap: tuple[int, ...]
BulgarianLangModel: tuple[int, ...]
Latin5BulgarianModel: _LangModelType
Win1251BulgarianModel: _LangModelType



from typing import Any

from ...types import Numeric
from .base import UUID, PGCompiler, PGDialect, PGIdentifierPreparer
from .hstore import HSTORE
from .json import JSON, JSONB

class _PGNumeric(Numeric):
    def bind_processor(self, dialect) -> None: ...
    def result_processor(self, dialect, coltype): ...

class _PGHStore(HSTORE):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGJSON(JSON):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGJSONB(JSONB):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGUUID(UUID):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGCompiler(PGCompiler):
    def visit_mod_binary(self, binary, operator, **kw): ...
    def post_process_text(self, text): ...

class _PGIdentifierPreparer(PGIdentifierPreparer): ...

class PGDialect_pygresql(PGDialect):
    driver: str
    supports_statement_cache: bool
    statement_compiler: Any
    preparer: Any
    @classmethod
    def dbapi(cls): ...
    colspecs: Any
    dbapi_version: Any
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    has_native_hstore: Any
    has_native_json: Any
    has_native_uuid: Any
    def __init__(self, **kwargs) -> None: ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = PGDialect_pygresql



from collections.abc import Callable
from typing import Any, TypeVar

_C = TypeVar("_C", bound=Callable[..., Any])

# Technically, the first argument of `_C` must be `Store`,
# but for now we leave it simple:
def initialised(func: _C) -> _C: ...
def trap(path_index: str | int) -> Callable[[_C], _C]: ...
def gen_password(length: int, symbols: bool = ...) -> str: ...
def copy_move(
    src: str, dst: str, force: bool = ..., move: bool = ..., interactive: bool = ..., verbose: bool = ...
) -> str | None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class PivotSource(Serialisable):
    tagname: str
    name: Any
    fmtId: Any
    extLst: Any
    __elements__: Any
    def __init__(self, name: Any | None = ..., fmtId: Any | None = ..., extLst: Any | None = ...) -> None: ...

class PivotFormat(Serialisable):
    tagname: str
    idx: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    TextBody: Any
    marker: Any
    dLbl: Any
    DataLabel: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        idx: int = ...,
        spPr: Any | None = ...,
        txPr: Any | None = ...,
        marker: Any | None = ...,
        dLbl: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class GraphicalProperties(Serialisable):
    tagname: str
    bwMode: Any
    xfrm: Any
    transform: Any
    custGeom: Any
    prstGeom: Any
    noFill: Any
    solidFill: Any
    gradFill: Any
    pattFill: Any
    ln: Any
    line: Any
    scene3d: Any
    sp3d: Any
    shape3D: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        bwMode: Any | None = ...,
        xfrm: Any | None = ...,
        noFill: Any | None = ...,
        solidFill: Any | None = ...,
        gradFill: Any | None = ...,
        pattFill: Any | None = ...,
        ln: Any | None = ...,
        scene3d: Any | None = ...,
        custGeom: Any | None = ...,
        prstGeom: Any | None = ...,
        sp3d: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...
